@@ -10,7 +10,9 @@ use std::{fs, io};
 use clap::{ArgGroup, Subcommand};
 use itertools::Itertools;
 use jj_lib::backend::{CommitId, ObjectId, TreeValue};
-use jj_lib::git::{self, parse_gitmodules, GitFetchError, GitPushError, GitRefUpdate};
+use jj_lib::git::{
+    self, parse_gitmodules, GitFetchError, GitFetchStats, GitPushError, GitRefUpdate,
+};
 use jj_lib::git_backend::GitBackend;
 use jj_lib::op_store::{BranchTarget, RefTarget};
 use jj_lib::refs::{classify_branch_push_action, BranchPushAction, BranchPushUpdate};
@@ -24,9 +26,9 @@ use jj_lib::workspace::Workspace;
 use maplit::hashset;
 
 use crate::cli_util::{
-    print_failed_git_export, resolve_multiple_nonempty_revsets, short_change_hash,
-    short_commit_hash, user_error, user_error_with_hint, CommandError, CommandHelper, RevisionArg,
-    WorkspaceCommandHelper,
+    print_failed_git_export, print_git_import_stats, resolve_multiple_nonempty_revsets,
+    short_change_hash, short_commit_hash, user_error, user_error_with_hint, CommandError,
+    CommandHelper, RevisionArg, WorkspaceCommandHelper,
 };
 use crate::commands::make_branch_term;
 use crate::progress::Progress;
@@ -185,7 +187,7 @@ pub struct GitSubmodulePrintGitmodulesArgs {
 fn get_git_repo(store: &Store) -> Result<git2::Repository, CommandError> {
     match store.backend_impl().downcast_ref::<GitBackend>() {
         None => Err(user_error("The repo is not backed by a git repo")),
-        Some(git_backend) => Ok(git_backend.git_repo_clone()),
+        Some(git_backend) => Ok(git_backend.open_git_repo()?),
     }
 }
 
@@ -330,7 +332,7 @@ fn cmd_git_fetch(
     ));
     let branches = args.branch.iter().map(|b| b.as_str()).collect_vec();
     for remote in remotes {
-        with_remote_callbacks(ui, |cb| {
+        let stats = with_remote_callbacks(ui, |cb| {
             git::fetch(
                 tx.mut_repo(),
                 &git_repo,
@@ -345,6 +347,7 @@ fn cmd_git_fetch(
             GitFetchError::InternalGitError(err) => map_git_error(err),
             _ => user_error(err.to_string()),
         })?;
+        print_git_import_stats(ui, &stats.import_stats)?;
     }
     tx.finish(ui)?;
     Ok(())
@@ -480,11 +483,12 @@ fn cmd_git_clone(
         }
     }
 
-    if let (mut workspace_command, git_repo, Some(default_branch)) = clone_result? {
+    let (mut workspace_command, git_repo, stats) = clone_result?;
+    if let Some(default_branch) = &stats.default_branch {
         let default_branch_target = workspace_command
             .repo()
             .view()
-            .get_remote_branch(&default_branch, "origin");
+            .get_remote_branch(default_branch, "origin");
         if let Some(commit_id) = default_branch_target.as_normal().cloned() {
             let mut checkout_tx =
                 workspace_command.start_transaction("check out git remote's default branch");
@@ -509,7 +513,7 @@ fn do_git_clone(
     colocate: bool,
     source: &str,
     wc_path: &Path,
-) -> Result<(WorkspaceCommandHelper, git2::Repository, Option<String>), CommandError> {
+) -> Result<(WorkspaceCommandHelper, git2::Repository, GitFetchStats), CommandError> {
     let (workspace, repo) = if colocate {
         let git_repo = git2::Repository::init(wc_path)?;
         add_to_git_exclude(ui, &git_repo)?;
@@ -524,7 +528,7 @@ fn do_git_clone(
     git_repo.remote(remote_name, source).unwrap();
     let mut fetch_tx = workspace_command.start_transaction("fetch from git remote into empty repo");
 
-    let maybe_default_branch = with_remote_callbacks(ui, |cb| {
+    let stats = with_remote_callbacks(ui, |cb| {
         git::fetch(
             fetch_tx.mut_repo(),
             &git_repo,
@@ -544,8 +548,9 @@ fn do_git_clone(
             unreachable!("we didn't provide any globs")
         }
     })?;
+    print_git_import_stats(ui, &stats.import_stats)?;
     fetch_tx.finish(ui)?;
-    Ok((workspace_command, git_repo, maybe_default_branch))
+    Ok((workspace_command, git_repo, stats))
 }
 
 fn with_remote_callbacks<T>(ui: &mut Ui, f: impl FnOnce(git::RemoteCallbacks<'_>) -> T) -> T {
@@ -905,7 +910,7 @@ fn cmd_git_push(
         {
             reasons.push("it has no author and/or committer set");
         }
-        if commit.tree()?.has_conflict() {
+        if commit.has_conflict()? {
             reasons.push("it has conflicts");
         }
         if !reasons.is_empty() {
@@ -974,7 +979,8 @@ fn cmd_git_push(
         ),
         _ => user_error(err.to_string()),
     })?;
-    git::import_refs(tx.mut_repo(), &git_repo, &command.settings().git_settings())?;
+    let stats = git::import_refs(tx.mut_repo(), &git_repo, &command.settings().git_settings())?;
+    print_git_import_stats(ui, &stats)?;
     tx.finish(ui)?;
     Ok(())
 }
@@ -1022,7 +1028,8 @@ fn cmd_git_import(
     let repo = workspace_command.repo();
     let git_repo = get_git_repo(repo.store())?;
     let mut tx = workspace_command.start_transaction("import git refs");
-    git::import_refs(tx.mut_repo(), &git_repo, &command.settings().git_settings())?;
+    let stats = git::import_refs(tx.mut_repo(), &git_repo, &command.settings().git_settings())?;
+    print_git_import_stats(ui, &stats)?;
     tx.finish(ui)?;
     Ok(())
 }
