@@ -35,7 +35,7 @@ use unicode_width::UnicodeWidthStr as _;
 
 use crate::cli_util::{CommandError, WorkspaceCommandHelper};
 use crate::formatter::Formatter;
-use crate::merge_tools::{self, ExternalMergeTool};
+use crate::merge_tools::{self, ExternalMergeTool, MergeTool};
 use crate::text_util;
 use crate::ui::Ui;
 
@@ -124,8 +124,13 @@ fn diff_formats_from_args(
     .collect_vec();
     if let Some(name) = &args.tool {
         let tool = merge_tools::get_tool_config(settings, name)?
-            .unwrap_or_else(|| ExternalMergeTool::with_program(name));
-        formats.push(DiffFormat::Tool(Box::new(tool)));
+            .unwrap_or_else(|| MergeTool::External(ExternalMergeTool::with_program(name)));
+        match tool {
+            MergeTool::Builtin => {}
+            MergeTool::External(tool) => {
+                formats.push(DiffFormat::Tool(Box::new(tool)));
+            }
+        }
     }
     Ok(formats)
 }
@@ -135,8 +140,13 @@ fn default_diff_format(settings: &UserSettings) -> Result<DiffFormat, config::Co
     if let Some(args) = config.get("ui.diff.tool").optional()? {
         // External "tool" overrides the internal "format" option.
         let tool = merge_tools::get_tool_config_from_args(settings, &args)?
-            .unwrap_or_else(|| ExternalMergeTool::with_diff_args(&args));
-        return Ok(DiffFormat::Tool(Box::new(tool)));
+            .unwrap_or_else(|| MergeTool::External(ExternalMergeTool::with_diff_args(&args)));
+        match tool {
+            MergeTool::Builtin => {}
+            MergeTool::External(tool) => {
+                return Ok(DiffFormat::Tool(Box::new(tool)));
+            }
+        }
     }
     let name = if let Some(name) = config.get_string("ui.diff.format").optional()? {
         name
