@@ -21,18 +21,21 @@ use config::ConfigError;
 use jj_lib::backend::MergedTreeId;
 use jj_lib::conflicts::extract_as_single_hunk;
 use jj_lib::gitignore::GitIgnoreFile;
+use jj_lib::matchers::Matcher;
 use jj_lib::merged_tree::MergedTree;
 use jj_lib::repo_path::RepoPath;
 use jj_lib::settings::{ConfigResultExt as _, UserSettings};
 use jj_lib::working_copy::SnapshotError;
 use thiserror::Error;
 
-use self::builtin::{edit_diff_builtin, BuiltinToolError};
+use self::builtin::{edit_diff_builtin, edit_merge_builtin, BuiltinToolError};
 use self::external::{edit_diff_external, DiffCheckoutError, ExternalToolError};
 pub use self::external::{generate_diff, ExternalMergeTool};
 use crate::config::CommandNameAndArgs;
 use crate::ui::Ui;
 
+const BUILTIN_EDITOR_NAME: &str = ":builtin";
+
 #[derive(Debug, Error)]
 pub enum DiffEditError {
     #[error(transparent)]
@@ -113,7 +116,10 @@ pub fn run_mergetool(
 
     let editor = get_merge_tool_from_settings(ui, settings)?;
     match editor {
-        MergeTool::Builtin => unimplemented!("run_mergetool with builtin mergetool"),
+        MergeTool::Builtin => {
+            let tree_id = edit_merge_builtin(tree, repo_path, content).map_err(Box::new)?;
+            Ok(tree_id)
+        }
         MergeTool::External(editor) => external::run_mergetool_external(
             &editor, file_merge, content, repo_path, conflict, tree,
         ),
@@ -124,6 +130,7 @@ pub fn edit_diff(
     ui: &Ui,
     left_tree: &MergedTree,
     right_tree: &MergedTree,
+    matcher: &dyn Matcher,
     instructions: &str,
     base_ignores: Arc<GitIgnoreFile>,
     settings: &UserSettings,
@@ -132,13 +139,14 @@ pub fn edit_diff(
     let editor = get_diff_editor_from_settings(ui, settings)?;
     match editor {
         MergeTool::Builtin => {
-            let tree_id = edit_diff_builtin(left_tree, right_tree).map_err(Box::new)?;
+            let tree_id = edit_diff_builtin(left_tree, right_tree, matcher).map_err(Box::new)?;
             Ok(tree_id)
         }
         MergeTool::External(editor) => edit_diff_external(
             editor,
             left_tree,
             right_tree,
+            matcher,
             instructions,
             base_ignores,
             settings,
@@ -152,11 +160,36 @@ pub enum MergeTool {
     External(ExternalMergeTool),
 }
 
+/// Finds the appropriate tool for diff editing or merges
+fn editor_args_from_settings(
+    ui: &Ui,
+    settings: &UserSettings,
+    key: &str,
+) -> Result<CommandNameAndArgs, ExternalToolError> {
+    // TODO: Make this configuration have a table of possible editors and detect the
+    // best one here.
+    if let Some(args) = settings.config().get(key).optional()? {
+        Ok(args)
+    } else {
+        let default_editor = BUILTIN_EDITOR_NAME;
+        writeln!(
+            ui.hint(),
+            "Using default editor '{default_editor}'; you can change this by setting {key}"
+        )
+        .map_err(ExternalToolError::Io)?;
+        Ok(default_editor.into())
+    }
+}
+
 /// Loads merge tool options from `[merge-tools.<name>]`.
 pub fn get_tool_config(
     settings: &UserSettings,
     name: &str,
-) -> Result<Option<ExternalMergeTool>, ConfigError> {
+) -> Result<Option<MergeTool>, ConfigError> {
+    if name == BUILTIN_EDITOR_NAME {
+        return Ok(Some(MergeTool::Builtin));
+    }
+
     const TABLE_KEY: &str = "merge-tools";
     let tools_table = settings.config().get_table(TABLE_KEY)?;
     if let Some(v) = tools_table.get(name) {
@@ -169,7 +202,7 @@ pub fn get_tool_config(
         if result.program.is_empty() {
             result.program.clone_from(&name.to_string());
         };
-        Ok(Some(result))
+        Ok(Some(MergeTool::External(result)))
     } else {
         Ok(None)
     }
@@ -180,7 +213,7 @@ pub fn get_tool_config(
 pub fn get_tool_config_from_args(
     settings: &UserSettings,
     args: &CommandNameAndArgs,
-) -> Result<Option<ExternalMergeTool>, ConfigError> {
+) -> Result<Option<MergeTool>, ConfigError> {
     match args {
         CommandNameAndArgs::String(name) => get_tool_config(settings, name),
         CommandNameAndArgs::Vec(_) | CommandNameAndArgs::Structured { .. } => Ok(None),
@@ -193,8 +226,8 @@ fn get_diff_editor_from_settings(
 ) -> Result<MergeTool, ExternalToolError> {
     let args = editor_args_from_settings(ui, settings, "ui.diff-editor")?;
     let editor = get_tool_config_from_args(settings, &args)?
-        .unwrap_or_else(|| ExternalMergeTool::with_edit_args(&args));
-    Ok(MergeTool::External(editor))
+        .unwrap_or_else(|| MergeTool::External(ExternalMergeTool::with_edit_args(&args)));
+    Ok(editor)
 }
 
 fn get_merge_tool_from_settings(
@@ -202,35 +235,15 @@ fn get_merge_tool_from_settings(
     settings: &UserSettings,
 ) -> Result<MergeTool, ExternalToolError> {
     let args = editor_args_from_settings(ui, settings, "ui.merge-editor")?;
-    let editor = get_tool_config_from_args(settings, &args)?
-        .unwrap_or_else(|| ExternalMergeTool::with_merge_args(&args));
-    if editor.merge_args.is_empty() {
-        Err(ExternalToolError::MergeArgsNotConfigured {
-            tool_name: args.to_string(),
-        })
-    } else {
-        Ok(MergeTool::External(editor))
-    }
-}
-
-/// Finds the appropriate tool for diff editing or merges
-fn editor_args_from_settings(
-    ui: &Ui,
-    settings: &UserSettings,
-    key: &str,
-) -> Result<CommandNameAndArgs, ExternalToolError> {
-    // TODO: Make this configuration have a table of possible editors and detect the
-    // best one here.
-    if let Some(args) = settings.config().get(key).optional()? {
-        Ok(args)
-    } else {
-        let default_editor = "meld";
-        writeln!(
-            ui.hint(),
-            "Using default editor '{default_editor}'; you can change this by setting {key}"
-        )
-        .map_err(ExternalToolError::Io)?;
-        Ok(default_editor.into())
+    let mergetool = get_tool_config_from_args(settings, &args)?
+        .unwrap_or_else(|| MergeTool::External(ExternalMergeTool::with_merge_args(&args)));
+    match mergetool {
+        MergeTool::External(mergetool) if mergetool.merge_args.is_empty() => {
+            Err(ExternalToolError::MergeArgsNotConfigured {
+                tool_name: args.to_string(),
+            })
+        }
+        mergetool => Ok(mergetool),
     }
 }
 
@@ -257,30 +270,7 @@ mod tests {
         };
 
         // Default
-        insta::assert_debug_snapshot!(get("").unwrap(), @r###"
-        External(
-            ExternalMergeTool {
-                program: "meld",
-                diff_args: [
-                    "$left",
-                    "$right",
-                ],
-                edit_args: [
-                    "$left",
-                    "$right",
-                ],
-                merge_args: [
-                    "$left",
-                    "$base",
-                    "$right",
-                    "-o",
-                    "$output",
-                    "--auto-merge",
-                ],
-                merge_tool_edits_conflict_markers: false,
-            },
-        )
-        "###);
+        insta::assert_debug_snapshot!(get("").unwrap(), @"Builtin");
 
         // Just program name, edit_args are filled by default
         insta::assert_debug_snapshot!(get(r#"ui.diff-editor = "my-diff""#).unwrap(), @r###"
@@ -429,30 +419,7 @@ mod tests {
         };
 
         // Default
-        insta::assert_debug_snapshot!(get("").unwrap(), @r###"
-        External(
-            ExternalMergeTool {
-                program: "meld",
-                diff_args: [
-                    "$left",
-                    "$right",
-                ],
-                edit_args: [
-                    "$left",
-                    "$right",
-                ],
-                merge_args: [
-                    "$left",
-                    "$base",
-                    "$right",
-                    "-o",
-                    "$output",
-                    "--auto-merge",
-                ],
-                merge_tool_edits_conflict_markers: false,
-            },
-        )
-        "###);
+        insta::assert_debug_snapshot!(get("").unwrap(), @"Builtin");
 
         // Just program name
         insta::assert_debug_snapshot!(get(r#"ui.merge-editor = "my-merge""#).unwrap_err(), @r###"
