@@ -95,6 +95,138 @@ fn test_git_colocated() {
     );
 }
 
+#[test]
+fn test_git_colocated_unborn_branch() {
+    let test_env = TestEnvironment::default();
+    let workspace_root = test_env.env_root().join("repo");
+    let git_repo = git2::Repository::init(&workspace_root).unwrap();
+
+    let add_file_to_index = |name: &str, data: &str| {
+        std::fs::write(workspace_root.join(name), data).unwrap();
+        let mut index = git_repo.index().unwrap();
+        index.add_path(Path::new(name)).unwrap();
+        index.write().unwrap();
+    };
+    let checkout_index = || {
+        let mut index = git_repo.index().unwrap();
+        index.read(true).unwrap(); // discard in-memory cache
+        git_repo.checkout_index(Some(&mut index), None).unwrap();
+    };
+
+    // Initially, HEAD isn't set.
+    test_env.jj_cmd_success(&workspace_root, &["init", "--git-repo", "."]);
+    assert!(git_repo.head().is_err());
+    assert_eq!(
+        git_repo.find_reference("HEAD").unwrap().symbolic_target(),
+        Some("refs/heads/master")
+    );
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  230dd059e1b059aefc0da06a2e5a7dbf22362f22
+    ◉  0000000000000000000000000000000000000000
+    "###);
+
+    // Stage some change, and check out root. This shouldn't clobber the HEAD.
+    add_file_to_index("file0", "");
+    insta::assert_snapshot!(
+        test_env.jj_cmd_success(&workspace_root, &["checkout", "root()"]), @r###"
+    Working copy now at: kkmpptxz fcdbbd73 (empty) (no description set)
+    Parent commit      : zzzzzzzz 00000000 (empty) (no description set)
+    Added 0 files, modified 0 files, removed 1 files
+    "###);
+    assert!(git_repo.head().is_err());
+    assert_eq!(
+        git_repo.find_reference("HEAD").unwrap().symbolic_target(),
+        Some("refs/heads/master")
+    );
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  fcdbbd731496cae17161cd6be9b6cf1f759655a8
+    │ ◉  1de814dbef9641cc6c5c80d2689b80778edcce09
+    ├─╯
+    ◉  0000000000000000000000000000000000000000
+    "###);
+    // Staged change shouldn't persist.
+    checkout_index();
+    insta::assert_snapshot!(test_env.jj_cmd_success(&workspace_root, &["status"]), @r###"
+    The working copy is clean
+    Working copy : kkmpptxz fcdbbd73 (empty) (no description set)
+    Parent commit: zzzzzzzz 00000000 (empty) (no description set)
+    "###);
+
+    // Stage some change, and create new HEAD. This shouldn't move the default
+    // branch.
+    add_file_to_index("file1", "");
+    insta::assert_snapshot!(test_env.jj_cmd_success(&workspace_root, &["new"]), @r###"
+    Working copy now at: royxmykx 76c60bf0 (empty) (no description set)
+    Parent commit      : kkmpptxz f8d5bc77 (no description set)
+    "###);
+    assert!(git_repo.head().unwrap().symbolic_target().is_none());
+    insta::assert_snapshot!(
+        git_repo.head().unwrap().peel_to_commit().unwrap().id().to_string(),
+        @"f8d5bc772d1147351fd6e8cea52a4f935d3b31e7"
+    );
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  76c60bf0a66dcbe74d74d58c23848d96f9e86e84
+    ◉  f8d5bc772d1147351fd6e8cea52a4f935d3b31e7 HEAD@git
+    │ ◉  1de814dbef9641cc6c5c80d2689b80778edcce09
+    ├─╯
+    ◉  0000000000000000000000000000000000000000
+    "###);
+    // Staged change shouldn't persist.
+    checkout_index();
+    insta::assert_snapshot!(test_env.jj_cmd_success(&workspace_root, &["status"]), @r###"
+    The working copy is clean
+    Working copy : royxmykx 76c60bf0 (empty) (no description set)
+    Parent commit: kkmpptxz f8d5bc77 (no description set)
+    "###);
+
+    // Assign the default branch. The branch is no longer "unborn".
+    test_env.jj_cmd_success(&workspace_root, &["branch", "set", "-r@-", "master"]);
+
+    // Stage some change, and check out root again. This should unset the HEAD.
+    // https://github.com/martinvonz/jj/issues/1495
+    add_file_to_index("file2", "");
+    insta::assert_snapshot!(
+        test_env.jj_cmd_success(&workspace_root, &["checkout", "root()"]), @r###"
+    Working copy now at: znkkpsqq 10dd328b (empty) (no description set)
+    Parent commit      : zzzzzzzz 00000000 (empty) (no description set)
+    Added 0 files, modified 0 files, removed 2 files
+    "###);
+    assert!(git_repo.head().is_err());
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  10dd328bb906e15890e55047740eab2812a3b2f7
+    │ ◉  2c576a57d2e6e8494616629cfdbb8fe5e3fea73b
+    │ ◉  f8d5bc772d1147351fd6e8cea52a4f935d3b31e7 master
+    ├─╯
+    │ ◉  1de814dbef9641cc6c5c80d2689b80778edcce09
+    ├─╯
+    ◉  0000000000000000000000000000000000000000
+    "###);
+    // Staged change shouldn't persist.
+    checkout_index();
+    insta::assert_snapshot!(test_env.jj_cmd_success(&workspace_root, &["status"]), @r###"
+    The working copy is clean
+    Working copy : znkkpsqq 10dd328b (empty) (no description set)
+    Parent commit: zzzzzzzz 00000000 (empty) (no description set)
+    "###);
+
+    // New snapshot and commit can be created after the HEAD got unset.
+    std::fs::write(workspace_root.join("file3"), "").unwrap();
+    insta::assert_snapshot!(test_env.jj_cmd_success(&workspace_root, &["new"]), @r###"
+    Working copy now at: wqnwkozp cab23370 (empty) (no description set)
+    Parent commit      : znkkpsqq 8f5b2638 (no description set)
+    "###);
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  cab233704a5c0b21bde070943055f22142fb2043
+    ◉  8f5b263819457712a2937428b9c58a2a84afbb1c HEAD@git
+    │ ◉  2c576a57d2e6e8494616629cfdbb8fe5e3fea73b
+    │ ◉  f8d5bc772d1147351fd6e8cea52a4f935d3b31e7 master
+    ├─╯
+    │ ◉  1de814dbef9641cc6c5c80d2689b80778edcce09
+    ├─╯
+    ◉  0000000000000000000000000000000000000000
+    "###);
+}
+
 #[test]
 fn test_git_colocated_export_branches_on_snapshot() {
     // Checks that we export branches that were changed only because the working
@@ -158,6 +290,8 @@ fn test_git_colocated_rebase_on_import() {
     git_repo.branch("master", &commit1, true).unwrap();
     git_repo.set_head("refs/heads/master").unwrap();
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    Abandoned 1 commits that are no longer reachable.
+    Done importing changes from the underlying Git repo.
     @  7f96185cfbe36341d0f9a86ebfaeab67a5922c7e
     ◉  4bcbeaba9a4b309c5f45a8807fbf5499b9714315 master HEAD@git add a file
     ◉  0000000000000000000000000000000000000000
@@ -176,7 +310,7 @@ fn test_git_colocated_branches() {
     @  3560559274ab431feea00b7b7e0b9250ecce951f bar
     │ ◉  1e6f0b403ed2ff9713b5d6b1dc601e4804250cda foo
     ├─╯
-    ◉  230dd059e1b059aefc0da06a2e5a7dbf22362f22 master HEAD@git
+    ◉  230dd059e1b059aefc0da06a2e5a7dbf22362f22 HEAD@git
     ◉  0000000000000000000000000000000000000000
     "###);
 
@@ -206,8 +340,10 @@ fn test_git_colocated_branches() {
         )
         .unwrap();
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    Abandoned 1 commits that are no longer reachable.
     Working copy now at: yqosqzyt 096dc80d (empty) (no description set)
     Parent commit      : qpvuntsm 230dd059 (empty) (no description set)
+    Done importing changes from the underlying Git repo.
     @  096dc80da67094fbaa6683e2a205dddffa31f9a8
     │ ◉  1e6f0b403ed2ff9713b5d6b1dc601e4804250cda master foo
     ├─╯
@@ -226,13 +362,12 @@ fn test_git_colocated_branch_forget() {
     test_env.jj_cmd_success(&workspace_root, &["branch", "set", "foo"]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  65b6b74e08973b88d38404430f119c8c79465250 foo
-    ◉  230dd059e1b059aefc0da06a2e5a7dbf22362f22 master HEAD@git
+    ◉  230dd059e1b059aefc0da06a2e5a7dbf22362f22 HEAD@git
     ◉  0000000000000000000000000000000000000000
     "###);
     let stdout = test_env.jj_cmd_success(&workspace_root, &["branch", "list"]);
     insta::assert_snapshot!(stdout, @r###"
     foo: rlvkpnrz 65b6b74e (empty) (no description set)
-    master: qpvuntsm 230dd059 (empty) (no description set)
     "###);
 
     let stdout = test_env.jj_cmd_success(&workspace_root, &["branch", "forget", "foo"]);
@@ -240,9 +375,7 @@ fn test_git_colocated_branch_forget() {
     // A forgotten branch is deleted in the git repo. For a detailed demo explaining
     // this, see `test_branch_forget_export` in `test_branch_command.rs`.
     let stdout = test_env.jj_cmd_success(&workspace_root, &["branch", "list"]);
-    insta::assert_snapshot!(stdout, @r###"
-    master: qpvuntsm 230dd059 (empty) (no description set)
-    "###);
+    insta::assert_snapshot!(stdout, @"");
 }
 
 #[test]
@@ -286,7 +419,7 @@ fn test_git_colocated_fetch_deleted_or_moved_branch() {
     ├─╯
     │ ◉  929e298ae9edf969b405a304c75c10457c47d52c B_to_delete B_to_delete
     ├─╯
-    ◉  a86754f975f953fa25da4265764adc0c62e9ce6b A master HEAD@git A
+    ◉  a86754f975f953fa25da4265764adc0c62e9ce6b A HEAD@git A
     ◉  0000000000000000000000000000000000000000
     "###);
 
@@ -294,14 +427,16 @@ fn test_git_colocated_fetch_deleted_or_moved_branch() {
     // Move branch C sideways
     test_env.jj_cmd_success(&origin_path, &["describe", "C_to_move", "-m", "moved C"]);
     let stdout = test_env.jj_cmd_success(&clone_path, &["git", "fetch"]);
-    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stdout, @r###"
+    Abandoned 2 commits that are no longer reachable.
+    "###);
     // "original C" and "B_to_delete" are abandoned, as the corresponding branches
     // were deleted or moved on the remote (#864)
     insta::assert_snapshot!(get_log_output(&test_env, &clone_path), @r###"
     ◉  04fd29df05638156b20044b3b6136b42abcb09ab C_to_move moved C
     │ @  0335878796213c3a701f1c9c34dcae242bee4131
     ├─╯
-    ◉  a86754f975f953fa25da4265764adc0c62e9ce6b A master HEAD@git A
+    ◉  a86754f975f953fa25da4265764adc0c62e9ce6b A HEAD@git A
     ◉  0000000000000000000000000000000000000000
     "###);
 }
@@ -313,13 +448,14 @@ fn test_git_colocated_external_checkout() {
     let git_repo = git2::Repository::init(&repo_path).unwrap();
     test_env.jj_cmd_success(&repo_path, &["init", "--git-repo=."]);
     test_env.jj_cmd_success(&repo_path, &["ci", "-m=A"]);
+    test_env.jj_cmd_success(&repo_path, &["branch", "set", "-r@-", "master"]);
     test_env.jj_cmd_success(&repo_path, &["new", "-m=B", "root()"]);
     test_env.jj_cmd_success(&repo_path, &["new"]);
 
     // Checked out anonymous branch
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
-    @  53637cd508ff02427dd78eca98f5b2450a6370ce
-    ◉  66f4d1806ae41bd604f69155dece64062a0056cf HEAD@git B
+    @  f8a23336e41840ed1757ef323402a770427dc89a
+    ◉  eccedddfa5152d99fc8ddd1081b375387a8a382a HEAD@git B
     │ ◉  a86754f975f953fa25da4265764adc0c62e9ce6b master A
     ├─╯
     ◉  0000000000000000000000000000000000000000
@@ -339,9 +475,10 @@ fn test_git_colocated_external_checkout() {
     // The old working-copy commit gets abandoned, but the whole branch should not
     // be abandoned. (#1042)
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
-    @  0521ce3b8c4e29aab79f3c750e2845dcbc4c3874
+    Done importing changes from the underlying Git repo.
+    @  adadbd65a794e2294962b3c3da9aada09fe1b472
     ◉  a86754f975f953fa25da4265764adc0c62e9ce6b master HEAD@git A
-    │ ◉  66f4d1806ae41bd604f69155dece64062a0056cf B
+    │ ◉  eccedddfa5152d99fc8ddd1081b375387a8a382a B
     ├─╯
     ◉  0000000000000000000000000000000000000000
     "###);
@@ -357,14 +494,14 @@ fn test_git_colocated_squash_undo() {
     // Test the setup
     insta::assert_snapshot!(get_log_output_divergence(&test_env, &repo_path), @r###"
     @  rlvkpnrzqnoo 8f71e3b6a3be
-    ◉  qpvuntsmwlqt a86754f975f9 A master HEAD@git
+    ◉  qpvuntsmwlqt a86754f975f9 A HEAD@git
     ◉  zzzzzzzzzzzz 000000000000
     "###);
 
     test_env.jj_cmd_success(&repo_path, &["squash"]);
     insta::assert_snapshot!(get_log_output_divergence(&test_env, &repo_path), @r###"
     @  zsuskulnrvyr f0c12b0396d9
-    ◉  qpvuntsmwlqt 2f376ea1478c A master HEAD@git
+    ◉  qpvuntsmwlqt 2f376ea1478c A HEAD@git
     ◉  zzzzzzzzzzzz 000000000000
     "###);
     test_env.jj_cmd_success(&repo_path, &["undo"]);
@@ -372,7 +509,7 @@ fn test_git_colocated_squash_undo() {
     // (#922)
     insta::assert_snapshot!(get_log_output_divergence(&test_env, &repo_path), @r###"
     @  rlvkpnrzqnoo 8f71e3b6a3be
-    ◉  qpvuntsmwlqt a86754f975f9 A master HEAD@git
+    ◉  qpvuntsmwlqt a86754f975f9 A HEAD@git
     ◉  zzzzzzzzzzzz 000000000000
     "###);
 }
