@@ -262,6 +262,7 @@ fn test_git_fetch_from_remote_named_git() {
     test_env.jj_cmd_success(&repo_path, &["git", "remote", "rename", "git", "bar"]);
     let stdout = test_env.jj_cmd_success(&repo_path, &["branch", "list"]);
     insta::assert_snapshot!(stdout, @r###"
+    Done importing changes from the underlying Git repo.
     git: mrylzrtu 76fc7466 message
     "###);
 }
@@ -382,6 +383,7 @@ fn create_trunk2_and_rebase_branches(test_env: &TestEnvironment, repo_path: &Pat
 #[test]
 fn test_git_fetch_all() {
     let test_env = TestEnvironment::default();
+    test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
     let source_git_repo_path = test_env.env_root().join("source");
     let _git_repo = git2::Repository::init(source_git_repo_path.clone()).unwrap();
 
@@ -403,7 +405,7 @@ fn test_git_fetch_all() {
     ├─╯
     │ ◉  359a9a02457d descr_for_a1 a1
     ├─╯
-    ◉  ff36dc55760e descr_for_trunk1 master trunk1
+    ◉  ff36dc55760e descr_for_trunk1 trunk1
     ◉  000000000000
     "###);
 
@@ -418,7 +420,6 @@ fn test_git_fetch_all() {
     a1: nknoxmzm 359a9a02 descr_for_a1
     a2: qkvnknrk decaa396 descr_for_a2
     b: vpupmnsl c7d4bdcb descr_for_b
-    master: zowqyktl ff36dc55 descr_for_trunk1
     trunk1: zowqyktl ff36dc55 descr_for_trunk1
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &target_jj_repo_path), @r###"
@@ -427,7 +428,7 @@ fn test_git_fetch_all() {
     ├─╯
     │ ◉  359a9a02457d descr_for_a1 a1
     ├─╯
-    ◉  ff36dc55760e descr_for_trunk1 master trunk1
+    ◉  ff36dc55760e descr_for_trunk1 trunk1
     │ @  230dd059e1b0
     ├─╯
     ◉  000000000000
@@ -444,7 +445,7 @@ fn test_git_fetch_all() {
     │ ◉  0424f6dfc1ff descr_for_a1 a1
     ├─╯
     @  8f1f14fbbf42 descr_for_trunk2 trunk2
-    ◉  ff36dc55760e descr_for_trunk1 master trunk1
+    ◉  ff36dc55760e descr_for_trunk1 trunk1
     ◉  000000000000
     "###);
     // Change a branch in the source repo as well, so that it becomes conflicted.
@@ -460,7 +461,7 @@ fn test_git_fetch_all() {
     ├─╯
     │ ◉  359a9a02457d descr_for_a1 a1
     ├─╯
-    ◉  ff36dc55760e descr_for_trunk1 master trunk1
+    ◉  ff36dc55760e descr_for_trunk1 trunk1
     │ @  230dd059e1b0
     ├─╯
     ◉  000000000000
@@ -470,10 +471,11 @@ fn test_git_fetch_all() {
     a2: qkvnknrk decaa396 descr_for_a2
     b: vpupmnsl 061eddbb new_descr_for_b_to_create_conflict
       @origin (ahead by 1 commits, behind by 1 commits): vpupmnsl c7d4bdcb descr_for_b
-    master: zowqyktl ff36dc55 descr_for_trunk1
     trunk1: zowqyktl ff36dc55 descr_for_trunk1
     "###);
-    insta::assert_snapshot!(test_env.jj_cmd_success(&target_jj_repo_path, &["git", "fetch"]), @"");
+    insta::assert_snapshot!(test_env.jj_cmd_success(&target_jj_repo_path, &["git", "fetch"]), @r###"
+    Abandoned 2 commits that are no longer reachable.
+    "###);
     insta::assert_snapshot!(get_branch_output(&test_env, &target_jj_repo_path), @r###"
     a1: quxllqov 0424f6df descr_for_a1
     a2: osusxwst 91e46b4b descr_for_a2
@@ -482,7 +484,6 @@ fn test_git_fetch_all() {
       + vpupmnsl 061eddbb new_descr_for_b_to_create_conflict
       + vktnwlsu babc4922 descr_for_b
       @origin (behind by 1 commits): vktnwlsu babc4922 descr_for_b
-    master: zowqyktl ff36dc55 descr_for_trunk1
     trunk1: zowqyktl ff36dc55 descr_for_trunk1
     trunk2: umznmzko 8f1f14fb descr_for_trunk2
     "###);
@@ -495,7 +496,7 @@ fn test_git_fetch_all() {
     ◉  8f1f14fbbf42 descr_for_trunk2 trunk2
     │ ◉  061eddbb43ab new_descr_for_b_to_create_conflict b??
     ├─╯
-    ◉  ff36dc55760e descr_for_trunk1 master trunk1
+    ◉  ff36dc55760e descr_for_trunk1 trunk1
     │ @  230dd059e1b0
     ├─╯
     ◉  000000000000
@@ -505,6 +506,7 @@ fn test_git_fetch_all() {
 #[test]
 fn test_git_fetch_some_of_many_branches() {
     let test_env = TestEnvironment::default();
+    test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
     let source_git_repo_path = test_env.env_root().join("source");
     let _git_repo = git2::Repository::init(source_git_repo_path.clone()).unwrap();
 
@@ -526,7 +528,7 @@ fn test_git_fetch_some_of_many_branches() {
     ├─╯
     │ ◉  359a9a02457d descr_for_a1 a1
     ├─╯
-    ◉  ff36dc55760e descr_for_trunk1 master trunk1
+    ◉  ff36dc55760e descr_for_trunk1 trunk1
     ◉  000000000000
     "###);
 
@@ -598,7 +600,7 @@ fn test_git_fetch_some_of_many_branches() {
     │ ◉  6f4e1c4dfe29 descr_for_a1 a1
     ├─╯
     @  09430ba04a82 descr_for_trunk2 trunk2
-    ◉  ff36dc55760e descr_for_trunk1 master trunk1
+    ◉  ff36dc55760e descr_for_trunk1 trunk1
     ◉  000000000000
     "###);
     // Change a branch in the source repo as well, so that it becomes conflicted.
@@ -623,7 +625,9 @@ fn test_git_fetch_some_of_many_branches() {
         &target_jj_repo_path,
         &["git", "fetch", "--branch", "b", "--branch", "a1"],
     );
-    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stdout, @r###"
+    Abandoned 1 commits that are no longer reachable.
+    "###);
     insta::assert_snapshot!(get_log_output(&test_env, &target_jj_repo_path), @r###"
     ◉  13ac032802f1 descr_for_b b?? b@origin
     │ ◉  6f4e1c4dfe29 descr_for_a1 a1
@@ -655,7 +659,9 @@ fn test_git_fetch_some_of_many_branches() {
         &target_jj_repo_path,
         &["git", "fetch", "--branch", "b", "--branch", "a*"],
     );
-    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stdout, @r###"
+    Abandoned 1 commits that are no longer reachable.
+    "###);
     insta::assert_snapshot!(get_log_output(&test_env, &target_jj_repo_path), @r###"
     ◉  010977d69c5b descr_for_a2 a2
     │ ◉  13ac032802f1 descr_for_b b?? b@origin
@@ -707,7 +713,7 @@ fn test_git_fetch_undo() {
     ├─╯
     │ ◉  359a9a02457d descr_for_a1 a1
     ├─╯
-    ◉  ff36dc55760e descr_for_trunk1 master trunk1
+    ◉  ff36dc55760e descr_for_trunk1 trunk1
     ◉  000000000000
     "###);
 
@@ -770,7 +776,7 @@ fn test_fetch_undo_what() {
     ├─╯
     │ ◉  359a9a02457d descr_for_a1 a1
     ├─╯
-    ◉  ff36dc55760e descr_for_trunk1 master trunk1
+    ◉  ff36dc55760e descr_for_trunk1 trunk1
     ◉  000000000000
     "###);
 
@@ -936,7 +942,7 @@ fn test_git_fetch_removed_branch() {
     ├─╯
     │ ◉  359a9a02457d descr_for_a1 a1
     ├─╯
-    ◉  ff36dc55760e descr_for_trunk1 master trunk1
+    ◉  ff36dc55760e descr_for_trunk1 trunk1
     ◉  000000000000
     "###);
 
@@ -949,7 +955,7 @@ fn test_git_fetch_removed_branch() {
     ├─╯
     │ ◉  359a9a02457d descr_for_a1 a1
     ├─╯
-    ◉  ff36dc55760e descr_for_trunk1 master trunk1
+    ◉  ff36dc55760e descr_for_trunk1 trunk1
     │ @  230dd059e1b0
     ├─╯
     ◉  000000000000
@@ -969,7 +975,7 @@ fn test_git_fetch_removed_branch() {
     ├─╯
     │ ◉  359a9a02457d descr_for_a1 a1
     ├─╯
-    ◉  ff36dc55760e descr_for_trunk1 master trunk1
+    ◉  ff36dc55760e descr_for_trunk1 trunk1
     │ @  230dd059e1b0
     ├─╯
     ◉  000000000000
@@ -977,12 +983,14 @@ fn test_git_fetch_removed_branch() {
 
     // Fetch branches a2 from origin, and check that it has been removed locally
     let stdout = test_env.jj_cmd_success(&target_jj_repo_path, &["git", "fetch", "--branch", "a2"]);
-    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stdout, @r###"
+    Abandoned 1 commits that are no longer reachable.
+    "###);
     insta::assert_snapshot!(get_log_output(&test_env, &target_jj_repo_path), @r###"
     ◉  c7d4bdcbc215 descr_for_b b
     │ ◉  359a9a02457d descr_for_a1 a1
     ├─╯
-    ◉  ff36dc55760e descr_for_trunk1 master trunk1
+    ◉  ff36dc55760e descr_for_trunk1 trunk1
     │ @  230dd059e1b0
     ├─╯
     ◉  000000000000
@@ -1013,7 +1021,7 @@ fn test_git_fetch_removed_parent_branch() {
     ├─╯
     │ ◉  359a9a02457d descr_for_a1 a1
     ├─╯
-    ◉  ff36dc55760e descr_for_trunk1 master trunk1
+    ◉  ff36dc55760e descr_for_trunk1 trunk1
     ◉  000000000000
     "###);
 
@@ -1026,7 +1034,7 @@ fn test_git_fetch_removed_parent_branch() {
     ├─╯
     │ ◉  359a9a02457d descr_for_a1 a1
     ├─╯
-    ◉  ff36dc55760e descr_for_trunk1 master trunk1
+    ◉  ff36dc55760e descr_for_trunk1 trunk1
     │ @  230dd059e1b0
     ├─╯
     ◉  000000000000
@@ -1044,7 +1052,9 @@ fn test_git_fetch_removed_parent_branch() {
             "git", "fetch", "--branch", "master", "--branch", "trunk1", "--branch", "a1",
         ],
     );
-    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stdout, @r###"
+    Abandoned 1 commits that are no longer reachable.
+    "###);
     insta::assert_snapshot!(get_log_output(&test_env, &target_jj_repo_path), @r###"
     ◉  c7d4bdcbc215 descr_for_b b
     │ ◉  decaa3966c83 descr_for_a2 a2
