@@ -68,8 +68,8 @@ fn test_no_subcommand() {
     let stdout = test_env.jj_cmd_success(test_env.env_root(), &["--version"]);
     let sanitized = stdout.replace(|c: char| c.is_ascii_hexdigit(), "?");
     assert!(
-        sanitized == "jj ?.?.?\n"
-            || sanitized == "jj ?.?.?-????????????????????????????????????????\n",
+        sanitized == "jj ?.??.?\n"
+            || sanitized == "jj ?.??.?-????????????????????????????????????????\n",
         "{sanitized}"
     );
 
@@ -430,7 +430,7 @@ fn test_help() {
       -v, --verbose                      Enable verbose logging
           --color <WHEN>                 When to colorize output (always, never, auto)
           --no-pager                     Disable the pager
-          --config-toml <TOML>           Additional configuration options
+          --config-toml <TOML>           Additional configuration options (can be repeated)
     "###);
 }
 
