@@ -314,6 +314,42 @@ fn test_log_with_or_without_diff() {
     "###);
 }
 
+#[test]
+fn test_log_null_terminate_multiline_descriptions() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_success(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    test_env.jj_cmd_success(
+        &repo_path,
+        &["commit", "-m", "commit 1 line 1", "-m", "commit 1 line 2"],
+    );
+    test_env.jj_cmd_success(
+        &repo_path,
+        &["commit", "-m", "commit 2 line 1", "-m", "commit 2 line 2"],
+    );
+    test_env.jj_cmd_success(
+        &repo_path,
+        &["describe", "-m", "commit 3 line 1", "-m", "commit 3 line 2"],
+    );
+
+    let stdout = test_env.jj_cmd_success(
+        &repo_path,
+        &[
+            "log",
+            "-r",
+            "~root()",
+            "-T",
+            r#"description ++ "\0""#,
+            "--no-graph",
+        ],
+    );
+    insta::assert_debug_snapshot!(
+        stdout,
+        @r###""commit 3 line 1\n\ncommit 3 line 2\n\0commit 2 line 1\n\ncommit 2 line 2\n\0commit 1 line 1\n\ncommit 1 line 2\n\0""###
+    )
+}
+
 #[test]
 fn test_log_shortest_accessors() {
     let test_env = TestEnvironment::default();
@@ -422,6 +458,26 @@ fn test_log_shortest_accessors() {
     "###);
 }
 
+#[test]
+fn test_log_bad_short_prefixes() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_success(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+    // Error on bad config of short prefixes
+    test_env.add_config(r#"revsets.short-prefixes = "!nval!d""#);
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["status"]);
+    insta::assert_snapshot!(stderr,
+        @r###"
+    Config error: Invalid `revsets.short-prefixes`:  --> 1:1
+      |
+    1 | !nval!d
+      | ^---
+      |
+      = expected <expression>
+    For help, see https://github.com/martinvonz/jj/blob/main/docs/config.md.
+    "###);
+}
+
 #[test]
 fn test_log_prefix_highlight_styled() {
     let test_env = TestEnvironment::default();
