@@ -21,6 +21,7 @@ pub mod common;
 #[test]
 fn test_templater_branches() {
     let test_env = TestEnvironment::default();
+    test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
 
     test_env.jj_cmd_success(test_env.env_root(), &["init", "--git", "origin"]);
     let origin_path = test_env.env_root().join("origin");
@@ -66,7 +67,9 @@ fn test_templater_branches() {
     let template = r#"commit_id.short() ++ " " ++ branches"#;
     let output = test_env.jj_cmd_success(&workspace_root, &["log", "-T", template]);
     insta::assert_snapshot!(output, @r###"
-    ◉  b1bb3766d584 branch3??
+    ◉  fed794e2ba44 branch3?? branch3@origin
+    │ ◉  b1bb3766d584 branch3??
+    ├─╯
     │ ◉  21c33875443e branch1*
     ├─╯
     │ @  a5b4d15489cc branch2* new-branch
@@ -279,6 +282,8 @@ fn test_templater_list_method() {
 
     insta::assert_snapshot!(render(r#""".lines().join("|")"#), @"");
     insta::assert_snapshot!(render(r#""a\nb\nc".lines().join("|")"#), @"a|b|c");
+    // Null separator
+    insta::assert_snapshot!(render(r#""a\nb\nc".lines().join("\0")"#), @"a\0b\0c");
     // Keyword as separator
     insta::assert_snapshot!(render(r#""a\nb\nc".lines().join(commit_id.short(2))"#), @"a00b00c");
 
