@@ -26,15 +26,18 @@ use tempfile::NamedTempFile;
 use thiserror::Error;
 
 use crate::backend::{BackendError, CommitId, ObjectId};
-use crate::git_backend::NO_GC_REF_NAMESPACE;
+use crate::commit::Commit;
+use crate::git_backend::GitBackend;
 use crate::op_store::{BranchTarget, RefTarget, RefTargetOptionExt};
 use crate::repo::{MutableRepo, Repo};
-use crate::revset;
+use crate::revset::{self, RevsetExpression};
 use crate::settings::GitSettings;
 use crate::view::{RefName, View};
 
 /// Reserved remote name for the backing Git repo.
 pub const REMOTE_NAME_FOR_LOCAL_GIT_REPO: &str = "git";
+/// Ref name used as a placeholder to unset HEAD without a commit.
+const UNBORN_ROOT_REF_NAME: &str = "refs/jj/root";
 
 #[derive(Error, Debug)]
 pub enum GitImportError {
@@ -59,6 +62,13 @@ pub enum GitImportError {
     InternalGitError(#[from] git2::Error),
 }
 
+/// Describes changes made by `import_refs()` or `fetch()`.
+#[derive(Clone, Debug, Eq, PartialEq)]
+pub struct GitImportStats {
+    /// Commits superseded by newly imported commits.
+    pub abandoned_commits: Vec<CommitId>,
+}
+
 fn parse_git_ref(ref_name: &str) -> Option<RefName> {
     if let Some(branch_name) = ref_name.strip_prefix("refs/heads/") {
         // Git CLI says 'HEAD' is not a valid branch name
@@ -172,18 +182,6 @@ pub fn get_local_git_tracking_branch<'a>(view: &'a View, branch: &str) -> &'a Re
     view.get_git_ref(&format!("refs/heads/{branch}"))
 }
 
-fn prevent_gc(git_repo: &git2::Repository, id: &CommitId) -> Result<(), git2::Error> {
-    // If multiple processes do git::import_refs() in parallel, this can fail to
-    // acquire a lock file even with force=true.
-    git_repo.reference(
-        &format!("{}{}", NO_GC_REF_NAMESPACE, id.hex()),
-        Oid::from_bytes(id.as_bytes()).unwrap(),
-        true,
-        "used by jj",
-    )?;
-    Ok(())
-}
-
 /// Reflect changes made in the underlying Git repo in the Jujutsu repo.
 ///
 /// This function detects conflicts (if both Git and JJ modified a branch) and
@@ -192,7 +190,7 @@ pub fn import_refs(
     mut_repo: &mut MutableRepo,
     git_repo: &git2::Repository,
     git_settings: &GitSettings,
-) -> Result<(), GitImportError> {
+) -> Result<GitImportStats, GitImportError> {
     import_some_refs(mut_repo, git_repo, git_settings, |_| true)
 }
 
@@ -205,7 +203,7 @@ pub fn import_some_refs(
     git_repo: &git2::Repository,
     git_settings: &GitSettings,
     git_ref_filter: impl Fn(&RefName) -> bool,
-) -> Result<(), GitImportError> {
+) -> Result<GitImportStats, GitImportError> {
     // TODO: Should this be a separate function? We may not always want to import
     // the Git HEAD (and add it to our set of heads).
     let old_git_head = mut_repo.view().git_head();
@@ -228,33 +226,44 @@ pub fn import_some_refs(
 
     // Import new heads
     let store = mut_repo.store();
+    // TODO: It might be better to obtain both git_repo and git_backend from
+    // mut_repo, and return error if the repo isn't backed by Git.
+    let git_backend = store.backend_impl().downcast_ref::<GitBackend>().unwrap();
+    // Bulk-import all reachable commits to reduce overhead of table merging.
+    let head_ids = itertools::chain(
+        &changed_git_head,
+        changed_git_refs.values().map(|(_, new_target)| new_target),
+    )
+    .flat_map(|target| target.added_ids());
+    let heads_imported = git_backend
+        .import_head_commits(head_ids, store.use_tree_conflict_format())
+        .is_ok();
     let mut head_commits = Vec::new();
+    let get_commit = |id| {
+        // If bulk-import failed, try again to find bad head or ref.
+        if !heads_imported {
+            git_backend.import_head_commits([id], store.use_tree_conflict_format())?;
+        }
+        store.get_commit(id)
+    };
     if let Some(new_head_target) = &changed_git_head {
         for id in new_head_target.added_ids() {
-            let commit = store
-                .get_commit(id)
-                .map_err(|err| GitImportError::MissingHeadTarget {
-                    id: id.clone(),
-                    err,
-                })?;
+            let commit = get_commit(id).map_err(|err| GitImportError::MissingHeadTarget {
+                id: id.clone(),
+                err,
+            })?;
             head_commits.push(commit);
         }
     }
     for (ref_name, (_, new_git_target)) in &changed_git_refs {
         for id in new_git_target.added_ids() {
-            let commit =
-                store
-                    .get_commit(id)
-                    .map_err(|err| GitImportError::MissingRefAncestor {
-                        ref_name: ref_name.to_string(),
-                        err,
-                    })?;
+            let commit = get_commit(id).map_err(|err| GitImportError::MissingRefAncestor {
+                ref_name: ref_name.to_string(),
+                err,
+            })?;
             head_commits.push(commit);
         }
     }
-    for commit in &head_commits {
-        prevent_gc(git_repo, commit.id())?;
-    }
     mut_repo.add_heads(&head_commits);
 
     // Apply the change that happened in git since last time we imported refs.
@@ -287,25 +296,37 @@ pub fn import_some_refs(
         .cloned()
         .collect_vec();
     if hidable_git_heads.is_empty() {
-        return Ok(());
+        let stats = GitImportStats {
+            abandoned_commits: vec![],
+        };
+        return Ok(stats);
     }
-    let pinned_heads = pinned_commit_ids(mut_repo.view()).cloned().collect_vec();
+    let pinned_heads = itertools::chain(
+        pinned_commit_ids(mut_repo.view()),
+        iter::once(mut_repo.store().root_commit_id()),
+    )
+    .cloned()
+    .collect_vec();
     // We could use mut_repo.record_rewrites() here but we know we only need to care
     // about abandoned commits for now. We may want to change this if we ever
     // add a way of preserving change IDs across rewrites by `git` (e.g. by
     // putting them in the commit message).
-    let abandoned_commits = revset::walk_revs(mut_repo, &hidable_git_heads, &pinned_heads)
+    let abandoned_expression = RevsetExpression::commits(pinned_heads)
+        .range(&RevsetExpression::commits(hidable_git_heads))
+        .intersection(&RevsetExpression::visible_heads().ancestors());
+    let abandoned_commits = revset::optimize(abandoned_expression)
+        .resolve(mut_repo)
+        .unwrap()
+        .evaluate(mut_repo)
         .unwrap()
         .iter()
         .collect_vec();
-    let root_commit_id = mut_repo.store().root_commit_id().clone();
-    for abandoned_commit in abandoned_commits {
-        if abandoned_commit != root_commit_id {
-            mut_repo.record_abandoned_commit(abandoned_commit);
-        }
+    for abandoned_commit in &abandoned_commits {
+        mut_repo.record_abandoned_commit(abandoned_commit.clone());
     }
 
-    Ok(())
+    let stats = GitImportStats { abandoned_commits };
+    Ok(stats)
 }
 
 /// Calculates diff of git refs to be imported.
@@ -357,17 +378,14 @@ fn diff_refs_to_import(
     Ok(changed_git_refs)
 }
 
-/// Commits referenced by local/remote branches, tags, or HEAD@git.
+/// Commits referenced by local branches, tags, or HEAD@git.
 ///
 /// On `import_refs()`, this is similar to collecting commits referenced by
 /// `view.git_refs()`. Main difference is that local branches can be moved by
 /// tracking remotes, and such mutation isn't applied to `view.git_refs()` yet.
 fn pinned_commit_ids(view: &View) -> impl Iterator<Item = &CommitId> {
-    let branch_ref_targets = view.branches().values().flat_map(|branch_target| {
-        iter::once(&branch_target.local_target).chain(branch_target.remote_targets.values())
-    });
     itertools::chain!(
-        branch_ref_targets,
+        view.local_branches().map(|(_, target)| target),
         view.tags().values(),
         iter::once(view.git_head()),
     )
@@ -532,10 +550,9 @@ fn diff_refs_to_export(
         let new_target = match &ref_name {
             RefName::LocalBranch(branch) => view.get_local_branch(branch),
             RefName::RemoteBranch { remote, branch } => {
-                // Currently, the only situation where this case occurs *and* new_target !=
-                // old_target is after a `jj branch forget`. So, in practice, for
-                // remote-tracking branches either `new_target == old_target` or
-                // `new_target == None`.
+                // There are two situations where remote-tracking branches get out of sync:
+                // 1. `jj branch forget`
+                // 2. `jj op undo`/`restore` in colocated repo
                 view.get_remote_branch(branch, remote)
             }
             _ => continue,
@@ -661,6 +678,43 @@ fn update_git_ref(
     Ok(())
 }
 
+/// Sets `HEAD@git` to the parent of the given working-copy commit and resets
+/// the Git index.
+pub fn reset_head(
+    mut_repo: &mut MutableRepo,
+    git_repo: &git2::Repository,
+    wc_commit: &Commit,
+) -> Result<(), git2::Error> {
+    let first_parent_id = &wc_commit.parent_ids()[0];
+    if first_parent_id != mut_repo.store().root_commit_id() {
+        let new_git_commit_id = Oid::from_bytes(first_parent_id.as_bytes()).unwrap();
+        let new_git_commit = git_repo.find_commit(new_git_commit_id)?;
+        git_repo.set_head_detached(new_git_commit_id)?;
+        git_repo.reset(new_git_commit.as_object(), git2::ResetType::Mixed, None)?;
+        mut_repo.set_git_head_target(RefTarget::normal(first_parent_id.clone()));
+    } else {
+        // Can't detach HEAD without a commit. Use placeholder ref to nullify the HEAD.
+        // We can't set_head() an arbitrary unborn ref, so use reference_symbolic()
+        // instead. Git CLI appears to deal with that. It would be nice if Git CLI
+        // couldn't create a commit without setting a valid branch name.
+        if mut_repo.git_head().is_present() {
+            match git_repo.find_reference(UNBORN_ROOT_REF_NAME) {
+                Ok(mut git_repo_ref) => git_repo_ref.delete()?,
+                Err(err) if err.code() == git2::ErrorCode::NotFound => {}
+                Err(err) => return Err(err),
+            }
+            git_repo.reference_symbolic("HEAD", UNBORN_ROOT_REF_NAME, true, "unset HEAD by jj")?;
+        }
+        // git_reset() of libgit2 requires a commit object. Do that manually.
+        let mut index = git_repo.index()?;
+        index.clear()?; // or read empty tree
+        index.write()?;
+        git_repo.cleanup_state()?;
+        mut_repo.set_git_head_target(RefTarget::absent());
+    }
+    Ok(())
+}
+
 #[derive(Debug, Error)]
 pub enum GitRemoteManagementError {
     #[error("No git remote named '{0}'")]
@@ -802,6 +856,15 @@ pub enum GitFetchError {
     InternalGitError(#[from] git2::Error),
 }
 
+/// Describes successful `fetch()` result.
+#[derive(Clone, Debug, Eq, PartialEq)]
+pub struct GitFetchStats {
+    /// Remote's default branch.
+    pub default_branch: Option<String>,
+    /// Changes made by the import.
+    pub import_stats: GitImportStats,
+}
+
 #[tracing::instrument(skip(mut_repo, git_repo, callbacks))]
 pub fn fetch(
     mut_repo: &mut MutableRepo,
@@ -810,7 +873,7 @@ pub fn fetch(
     branch_name_globs: Option<&[&str]>,
     callbacks: RemoteCallbacks<'_>,
     git_settings: &GitSettings,
-) -> Result<Option<String>, GitFetchError> {
+) -> Result<GitFetchStats, GitFetchError> {
     let branch_name_filter = {
         let regex = if let Some(globs) = branch_name_globs {
             let result = regex::RegexSet::new(
@@ -853,9 +916,7 @@ pub fn fetch(
     // help branch forget`.
     let nonempty_branches: HashSet<_> = mut_repo
         .view()
-        .branches()
-        .iter()
-        .filter(|&(_branch, target)| target.local_target.is_present())
+        .local_branches()
         .map(|(branch, _target)| branch.to_owned())
         .collect();
     // TODO: Inform the user if the export failed? In most cases, export is not
@@ -916,15 +977,20 @@ pub fn fetch(
     tracing::debug!("remote.disconnect");
     remote.disconnect()?;
 
-    // `import_some_refs` will import the remote-tracking branches into the jj repo
-    // and update jj's local branches.
+    // Import the remote-tracking branches into the jj repo and update jj's
+    // local branches. We also import local tags since remote tags should have
+    // been merged by Git.
     tracing::debug!("import_refs");
-    import_some_refs(mut_repo, git_repo, git_settings, |ref_name| {
+    let import_stats = import_some_refs(mut_repo, git_repo, git_settings, |ref_name| {
         to_remote_branch(ref_name, remote_name)
             .map(&branch_name_filter)
-            .unwrap_or(false)
+            .unwrap_or_else(|| matches!(ref_name, RefName::Tag(_)))
     })?;
-    Ok(default_branch)
+    let stats = GitFetchStats {
+        default_branch,
+        import_stats,
+    };
+    Ok(stats)
 }
 
 #[derive(Error, Debug, PartialEq)]
