@@ -180,7 +180,8 @@ impl MergedTree {
         match self {
             MergedTree::Legacy(tree) => match tree.value(basename) {
                 Some(TreeValue::Conflict(conflict_id)) => {
-                    let conflict = tree.store().read_conflict(tree.dir(), conflict_id).unwrap();
+                    let path = tree.dir().join(basename);
+                    let conflict = tree.store().read_conflict(&path, conflict_id).unwrap();
                     MergedTreeValue::Conflict(conflict)
                 }
                 other => MergedTreeValue::Resolved(other),
@@ -391,7 +392,15 @@ impl MergedTree {
             // a resolved merge. However, that function will always preserve the arity of
             // conflicts it cannot resolve. So we simplify the conflict again
             // here to possibly reduce a complex conflict to a simpler one.
-            Ok(MergedTree::Merge(tree.simplify()))
+            let tree = tree.simplify();
+            // If debug assertions are enabled, check that the merge was idempotent. In
+            // particular,  that this last simplification doesn't enable further automatic
+            // resolutions
+            if cfg!(debug_assertions) {
+                let re_merged = merge_trees(&tree).unwrap();
+                debug_assert_eq!(re_merged, tree);
+            }
+            Ok(MergedTree::Merge(tree))
         }
     }
 }
@@ -418,7 +427,8 @@ fn merge_trees(merge: &Merge<Tree>) -> Result<Merge<Tree>, TreeMergeError> {
     let mut conflicts = vec![];
     for basename in all_tree_conflict_names(merge) {
         let path_merge = merge.map(|tree| tree.value(basename).cloned());
-        let path_merge = merge_tree_values(store, dir, path_merge)?;
+        let path = dir.join(basename);
+        let path_merge = merge_tree_values(store, &path, path_merge)?;
         match path_merge.into_resolved() {
             Ok(value) => {
                 new_tree.set_or_remove(basename, value);
