@@ -433,34 +433,27 @@ pub fn try_resolve_file_conflict(
             executable,
         }));
     }
-    let mut removed_contents = vec![];
-    let mut added_contents = vec![];
-    for &file_id in file_id_conflict.removes() {
-        let mut content = vec![];
-        store
-            .read_file(filename, file_id)?
-            .read_to_end(&mut content)
-            .map_err(|err| TreeMergeError::ReadError {
-                source: err,
-                file_id: file_id.clone(),
-            })?;
-        removed_contents.push(content);
-    }
-    for &file_id in file_id_conflict.adds() {
-        let mut content = vec![];
-        store
-            .read_file(filename, file_id)?
-            .read_to_end(&mut content)
-            .map_err(|err| TreeMergeError::ReadError {
-                source: err,
-                file_id: file_id.clone(),
-            })?;
-        added_contents.push(content);
-    }
-    let merge_result = files::merge(
-        &removed_contents.iter().map(Vec::as_slice).collect_vec(),
-        &added_contents.iter().map(Vec::as_slice).collect_vec(),
-    );
+
+    // Simplify the conflict for two reasons:
+    // 1. Avoid reading unchanged file contents
+    // 2. The simplified conflict can sometimes be resolved when the unsimplfied one
+    //    cannot
+    let file_id_conflict = file_id_conflict.simplify();
+
+    let contents: Merge<Vec<u8>> =
+        file_id_conflict.try_map(|&file_id| -> Result<Vec<u8>, TreeMergeError> {
+            let mut content = vec![];
+            store
+                .read_file(filename, file_id)?
+                .read_to_end(&mut content)
+                .map_err(|err| TreeMergeError::ReadError {
+                    source: err,
+                    file_id: file_id.clone(),
+                })?;
+            Ok(content)
+        })?;
+    let slices = contents.map(|content| content.as_slice());
+    let merge_result = files::merge(slices);
     match merge_result {
         MergeResult::Resolved(merged_content) => {
             let id = store.write_file(filename, &mut merged_content.0.as_slice())?;
