@@ -14,10 +14,11 @@
 
 use std::path::Path;
 
+use itertools::Itertools;
 use jj_lib::repo::{Repo, StoreFactories};
 use jj_lib::workspace::Workspace;
 use test_case::test_case;
-use testutils::{create_random_commit, load_repo_at_head, TestWorkspace};
+use testutils::{create_random_commit, load_repo_at_head, TestRepoBackend, TestWorkspace};
 
 fn copy_directory(src: &Path, dst: &Path) {
     std::fs::create_dir(dst).ok();
@@ -34,67 +35,73 @@ fn copy_directory(src: &Path, dst: &Path) {
 }
 
 fn merge_directories(left: &Path, base: &Path, right: &Path, output: &Path) {
-    std::fs::create_dir(output).ok();
+    std::fs::create_dir(output).unwrap();
     let mut sub_dirs = vec![];
     // Walk the left side and copy to the output
-    for entry in std::fs::read_dir(left).unwrap() {
-        let path = entry.unwrap().path();
-        let base_name = path.file_name().unwrap();
-        let child_left = left.join(base_name);
-        let child_output = output.join(base_name);
-        if child_left.is_dir() {
-            sub_dirs.push(base_name.to_os_string());
-        } else {
-            std::fs::copy(&child_left, child_output).unwrap();
+    if left.exists() {
+        for entry in std::fs::read_dir(left).unwrap() {
+            let path = entry.unwrap().path();
+            let base_name = path.file_name().unwrap();
+            let child_left = left.join(base_name);
+            let child_output = output.join(base_name);
+            if child_left.is_dir() {
+                sub_dirs.push(base_name.to_os_string());
+            } else {
+                std::fs::copy(&child_left, child_output).unwrap();
+            }
         }
     }
     // Walk the base and find files removed in the right side, then remove them in
     // the output
-    for entry in std::fs::read_dir(base).unwrap() {
-        let path = entry.unwrap().path();
-        let base_name = path.file_name().unwrap();
-        let child_base = base.join(base_name);
-        let child_right = right.join(base_name);
-        let child_output = output.join(base_name);
-        if child_base.is_dir() {
-            sub_dirs.push(base_name.to_os_string());
-        } else if !child_right.exists() {
-            std::fs::remove_file(child_output).ok();
+    if base.exists() {
+        for entry in std::fs::read_dir(base).unwrap() {
+            let path = entry.unwrap().path();
+            let base_name = path.file_name().unwrap();
+            let child_base = base.join(base_name);
+            let child_right = right.join(base_name);
+            let child_output = output.join(base_name);
+            if child_base.is_dir() {
+                sub_dirs.push(base_name.to_os_string());
+            } else if !child_right.exists() {
+                std::fs::remove_file(child_output).ok();
+            }
         }
     }
     // Walk the right side and find files added in the right side, then add them in
     // the output
-    for entry in std::fs::read_dir(right).unwrap() {
-        let path = entry.unwrap().path();
-        let base_name = path.file_name().unwrap();
-        let child_base = base.join(base_name);
-        let child_right = right.join(base_name);
-        let child_output = output.join(base_name);
-        if child_right.is_dir() {
-            sub_dirs.push(base_name.to_os_string());
-        } else if !child_base.exists() {
-            // This overwrites the left side if that's been written. That's fine, since the
-            // point of the test is that it should be okay for either side to win.
-            std::fs::copy(&child_right, child_output).unwrap();
+    if right.exists() {
+        for entry in std::fs::read_dir(right).unwrap() {
+            let path = entry.unwrap().path();
+            let base_name = path.file_name().unwrap();
+            let child_base = base.join(base_name);
+            let child_right = right.join(base_name);
+            let child_output = output.join(base_name);
+            if child_right.is_dir() {
+                sub_dirs.push(base_name.to_os_string());
+            } else if !child_base.exists() {
+                // This overwrites the left side if that's been written. That's fine, since the
+                // point of the test is that it should be okay for either side to win.
+                std::fs::copy(&child_right, child_output).unwrap();
+            }
         }
     }
     // Do the merge in subdirectories
-    for base_name in sub_dirs {
-        let child_base = base.join(&base_name);
-        let child_right = right.join(&base_name);
-        let child_left = left.join(&base_name);
-        let child_output = output.join(&base_name);
+    for base_name in sub_dirs.iter().sorted().dedup() {
+        let child_base = base.join(base_name);
+        let child_right = right.join(base_name);
+        let child_left = left.join(base_name);
+        let child_output = output.join(base_name);
         merge_directories(&child_left, &child_base, &child_right, &child_output);
     }
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_bad_locking_children(use_git: bool) {
+#[test_case(TestRepoBackend::Local; "local backend")]
+#[test_case(TestRepoBackend::Git; "git backend")]
+fn test_bad_locking_children(backend: TestRepoBackend) {
     // Test that two new commits created on separate machines are both visible (not
     // lost due to lack of locking)
     let settings = testutils::user_settings();
-    let test_workspace = TestWorkspace::init(&settings, use_git);
+    let test_workspace = TestWorkspace::init_with_backend(&settings, backend);
     let repo = &test_workspace.repo;
     let workspace_root = test_workspace.workspace.workspace_root();
 
@@ -106,10 +113,10 @@ fn test_bad_locking_children(use_git: bool) {
     tx.commit();
 
     // Simulate a write of a commit that happens on one machine
-    let machine1_root = testutils::new_temp_dir();
-    copy_directory(workspace_root, machine1_root.path());
+    let machine1_root = test_workspace.root_dir().join("machine1");
+    copy_directory(workspace_root, &machine1_root);
     let machine1_workspace =
-        Workspace::load(&settings, machine1_root.path(), &StoreFactories::default()).unwrap();
+        Workspace::load(&settings, &machine1_root, &StoreFactories::default()).unwrap();
     let machine1_repo = machine1_workspace
         .repo_loader()
         .load_at_head(&settings)
@@ -122,10 +129,10 @@ fn test_bad_locking_children(use_git: bool) {
     machine1_tx.commit();
 
     // Simulate a write of a commit that happens on another machine
-    let machine2_root = testutils::new_temp_dir();
-    copy_directory(workspace_root, machine2_root.path());
+    let machine2_root = test_workspace.root_dir().join("machine2");
+    copy_directory(workspace_root, &machine2_root);
     let machine2_workspace =
-        Workspace::load(&settings, machine2_root.path(), &StoreFactories::default()).unwrap();
+        Workspace::load(&settings, &machine2_root, &StoreFactories::default()).unwrap();
     let machine2_repo = machine2_workspace
         .repo_loader()
         .load_at_head(&settings)
@@ -139,15 +146,10 @@ fn test_bad_locking_children(use_git: bool) {
 
     // Simulate that the distributed file system now has received the changes from
     // both machines
-    let merged_path = testutils::new_temp_dir();
-    merge_directories(
-        machine1_root.path(),
-        workspace_root,
-        machine2_root.path(),
-        merged_path.path(),
-    );
+    let merged_path = test_workspace.root_dir().join("merged");
+    merge_directories(&machine1_root, workspace_root, &machine2_root, &merged_path);
     let merged_workspace =
-        Workspace::load(&settings, merged_path.path(), &StoreFactories::default()).unwrap();
+        Workspace::load(&settings, &merged_path, &StoreFactories::default()).unwrap();
     let merged_repo = merged_workspace
         .repo_loader()
         .load_at_head(&settings)
@@ -159,14 +161,14 @@ fn test_bad_locking_children(use_git: bool) {
     assert_eq!(op.parents.len(), 2);
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_bad_locking_interrupted(use_git: bool) {
+#[test_case(TestRepoBackend::Local ; "local backend")]
+#[test_case(TestRepoBackend::Git ; "git backend")]
+fn test_bad_locking_interrupted(backend: TestRepoBackend) {
     // Test that an interrupted update of the op-heads resulting in on op-head
     // that's a descendant of the other is resolved without creating a new
     // operation.
     let settings = testutils::user_settings();
-    let test_workspace = TestWorkspace::init(&settings, use_git);
+    let test_workspace = TestWorkspace::init_with_backend(&settings, backend);
     let repo = &test_workspace.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -181,8 +183,8 @@ fn test_bad_locking_interrupted(use_git: bool) {
     // operation and then copying that back afterwards, leaving the existing
     // op-head(s) in place.
     let op_heads_dir = repo.repo_path().join("op_heads");
-    let backup_path = testutils::new_temp_dir();
-    copy_directory(&op_heads_dir, backup_path.path());
+    let backup_path = test_workspace.root_dir().join("backup");
+    copy_directory(&op_heads_dir, &backup_path);
     let mut tx = repo.start_transaction(&settings, "test");
     create_random_commit(tx.mut_repo(), &settings)
         .set_parents(vec![initial.id().clone()])
@@ -190,7 +192,7 @@ fn test_bad_locking_interrupted(use_git: bool) {
         .unwrap();
     let op_id = tx.commit().operation().id().clone();
 
-    copy_directory(backup_path.path(), &op_heads_dir);
+    copy_directory(&backup_path, &op_heads_dir);
     // Reload the repo and check that only the new head is present.
     let reloaded_repo = load_repo_at_head(&settings, repo.repo_path());
     assert_eq!(reloaded_repo.op_id(), &op_id);
