@@ -35,10 +35,12 @@ use jj_lib::op_store::{BranchTarget, RefTarget};
 use jj_lib::repo::{MutableRepo, ReadonlyRepo, Repo};
 use jj_lib::settings::{GitSettings, UserSettings};
 use jj_lib::view::RefName;
+use jj_lib::workspace::Workspace;
 use maplit::{btreemap, hashset};
 use tempfile::TempDir;
 use testutils::{
     commit_transactions, create_random_commit, load_repo_at_head, write_random_commit, TestRepo,
+    TestRepoBackend,
 };
 
 fn empty_git_commit<'r>(
@@ -70,19 +72,22 @@ fn git_id(commit: &Commit) -> Oid {
     Oid::from_bytes(commit.id().as_bytes()).unwrap()
 }
 
-fn get_git_repo(repo: &Arc<ReadonlyRepo>) -> git2::Repository {
+fn get_git_backend(repo: &Arc<ReadonlyRepo>) -> &GitBackend {
     repo.store()
         .backend_impl()
         .downcast_ref::<GitBackend>()
         .unwrap()
-        .git_repo_clone()
+}
+
+fn get_git_repo(repo: &Arc<ReadonlyRepo>) -> git2::Repository {
+    get_git_backend(repo).open_git_repo().unwrap()
 }
 
 #[test]
 fn test_import_refs() {
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_repo.repo;
     let git_repo = get_git_repo(repo);
 
@@ -101,11 +106,12 @@ fn test_import_refs() {
 
     let git_repo = get_git_repo(repo);
     let mut tx = repo.start_transaction(&settings, "test");
-    git::import_refs(tx.mut_repo(), &git_repo, &git_settings).unwrap();
+    let stats = git::import_refs(tx.mut_repo(), &git_repo, &git_settings).unwrap();
     tx.mut_repo().rebase_descendants(&settings).unwrap();
     let repo = tx.commit();
     let view = repo.view();
 
+    assert!(stats.abandoned_commits.is_empty());
     let expected_heads = hashset! {
         jj_id(&commit3),
         jj_id(&commit4),
@@ -182,7 +188,7 @@ fn test_import_refs() {
 fn test_import_refs_reimport() {
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
-    let test_workspace = TestRepo::init(true);
+    let test_workspace = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_workspace.repo;
     let git_repo = get_git_repo(repo);
 
@@ -197,10 +203,11 @@ fn test_import_refs_reimport() {
         .unwrap();
 
     let mut tx = repo.start_transaction(&settings, "test");
-    git::import_refs(tx.mut_repo(), &git_repo, &git_settings).unwrap();
+    let stats = git::import_refs(tx.mut_repo(), &git_repo, &git_settings).unwrap();
     tx.mut_repo().rebase_descendants(&settings).unwrap();
     let repo = tx.commit();
 
+    assert!(stats.abandoned_commits.is_empty());
     let expected_heads = hashset! {
             jj_id(&commit3),
             jj_id(&commit4),
@@ -224,10 +231,18 @@ fn test_import_refs_reimport() {
     let repo = tx.commit();
 
     let mut tx = repo.start_transaction(&settings, "test");
-    git::import_refs(tx.mut_repo(), &git_repo, &git_settings).unwrap();
+    let stats = git::import_refs(tx.mut_repo(), &git_repo, &git_settings).unwrap();
     tx.mut_repo().rebase_descendants(&settings).unwrap();
     let repo = tx.commit();
 
+    assert_eq!(
+        // The order is unstable just because we import heads from Git repo.
+        HashSet::from_iter(stats.abandoned_commits),
+        hashset! {
+            jj_id(&commit4),
+            jj_id(&commit3),
+        },
+    );
     let view = repo.view();
     let expected_heads = hashset! {
             jj_id(&commit5),
@@ -274,7 +289,7 @@ fn test_import_refs_reimport_head_removed() {
     // Test that re-importing refs doesn't cause a deleted head to come back
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_repo.repo;
     let git_repo = get_git_repo(repo);
 
@@ -299,7 +314,7 @@ fn test_import_refs_reimport_git_head_counts() {
     // descendant of it), we still keep it alive.
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_repo.repo;
     let git_repo = get_git_repo(repo);
 
@@ -327,7 +342,7 @@ fn test_import_refs_reimport_git_head_without_ref() {
     // Simulate external `git checkout` in colocated repo, from anonymous branch.
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_repo.repo;
     let git_repo = get_git_repo(repo);
 
@@ -361,7 +376,7 @@ fn test_import_refs_reimport_git_head_with_moved_ref() {
     // Simulate external history rewriting in colocated repo.
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_repo.repo;
     let git_repo = get_git_repo(repo);
 
@@ -397,7 +412,7 @@ fn test_import_refs_reimport_git_head_with_moved_ref() {
 fn test_import_refs_reimport_with_deleted_remote_ref() {
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
-    let test_workspace = TestRepo::init(true);
+    let test_workspace = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_workspace.repo;
     let git_repo = get_git_repo(repo);
 
@@ -486,7 +501,7 @@ fn test_import_refs_reimport_with_deleted_remote_ref() {
 fn test_import_refs_reimport_with_moved_remote_ref() {
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
-    let test_workspace = TestRepo::init(true);
+    let test_workspace = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_workspace.repo;
     let git_repo = get_git_repo(repo);
 
@@ -602,7 +617,7 @@ fn test_import_refs_reimport_git_head_with_fixed_ref() {
     // Simulate external `git checkout` in colocated repo, from named branch.
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_repo.repo;
     let git_repo = get_git_repo(repo);
 
@@ -637,7 +652,7 @@ fn test_import_refs_reimport_all_from_root_removed() {
     // we abandon the whole stack, but not including the root commit.
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_repo.repo;
     let git_repo = get_git_repo(repo);
 
@@ -663,7 +678,7 @@ fn test_import_refs_reimport_all_from_root_removed() {
 fn test_import_refs_reimport_conflicted_remote_branch() {
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_repo.repo;
     let git_repo = get_git_repo(repo);
 
@@ -706,7 +721,7 @@ fn test_import_refs_reimport_conflicted_remote_branch() {
 fn test_import_refs_reserved_remote_name() {
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_repo.repo;
     let git_repo = get_git_repo(repo);
 
@@ -721,7 +736,7 @@ fn test_import_refs_reserved_remote_name() {
 fn test_import_some_refs() {
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
-    let test_workspace = TestRepo::init(true);
+    let test_workspace = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_workspace.repo;
     let git_repo = get_git_repo(repo);
 
@@ -963,7 +978,7 @@ fn test_import_refs_empty_git_repo() {
 fn test_import_refs_missing_git_commit() {
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
-    let test_workspace = TestRepo::init(true);
+    let test_workspace = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_workspace.repo;
     let git_repo = get_git_repo(repo);
 
@@ -1558,6 +1573,64 @@ fn test_export_reexport_transitions() {
     );
 }
 
+#[test]
+fn test_reset_head_to_root() {
+    // Create colocated workspace
+    let settings = testutils::user_settings();
+    let temp_dir = testutils::new_temp_dir();
+    let workspace_root = temp_dir.path().join("repo");
+    let git_repo = git2::Repository::init(&workspace_root).unwrap();
+    let (_workspace, repo) =
+        Workspace::init_external_git(&settings, &workspace_root, &workspace_root.join(".git"))
+            .unwrap();
+
+    let mut tx = repo.start_transaction(&settings, "test");
+    let mut_repo = tx.mut_repo();
+
+    let root_commit_id = repo.store().root_commit_id();
+    let tree_id = repo.store().empty_merged_tree_id();
+    let commit1 = mut_repo
+        .new_commit(&settings, vec![root_commit_id.clone()], tree_id.clone())
+        .write()
+        .unwrap();
+    let commit2 = mut_repo
+        .new_commit(&settings, vec![commit1.id().clone()], tree_id.clone())
+        .write()
+        .unwrap();
+
+    // Set Git HEAD to commit2's parent (i.e. commit1)
+    git::reset_head(tx.mut_repo(), &git_repo, &commit2).unwrap();
+    assert!(git_repo.head().is_ok());
+    assert_eq!(
+        tx.mut_repo().git_head(),
+        RefTarget::normal(commit1.id().clone())
+    );
+
+    // Set Git HEAD back to root
+    git::reset_head(tx.mut_repo(), &git_repo, &commit1).unwrap();
+    assert!(git_repo.head().is_err());
+    assert!(tx.mut_repo().git_head().is_absent());
+
+    // Move placeholder ref as if new commit were created by git
+    git_repo
+        .reference("refs/jj/root", git_id(&commit1), false, "")
+        .unwrap();
+    git::reset_head(tx.mut_repo(), &git_repo, &commit2).unwrap();
+    assert!(git_repo.head().is_ok());
+    assert_eq!(
+        tx.mut_repo().git_head(),
+        RefTarget::normal(commit1.id().clone())
+    );
+    assert!(git_repo.find_reference("refs/jj/root").is_ok());
+
+    // Set Git HEAD back to root
+    git::reset_head(tx.mut_repo(), &git_repo, &commit1).unwrap();
+    assert!(git_repo.head().is_err());
+    assert!(tx.mut_repo().git_head().is_absent());
+    // The placeholder ref should be deleted
+    assert!(git_repo.find_reference("refs/jj/root").is_err());
+}
+
 #[test]
 fn test_init() {
     let settings = testutils::user_settings();
@@ -1595,7 +1668,7 @@ fn test_fetch_empty_repo() {
     let mut tx = test_data
         .repo
         .start_transaction(&test_data.settings, "test");
-    let default_branch = git::fetch(
+    let stats = git::fetch(
         tx.mut_repo(),
         &test_data.git_repo,
         "origin",
@@ -1605,7 +1678,8 @@ fn test_fetch_empty_repo() {
     )
     .unwrap();
     // No default branch and no refs
-    assert_eq!(default_branch, None);
+    assert_eq!(stats.default_branch, None);
+    assert!(stats.import_stats.abandoned_commits.is_empty());
     assert_eq!(*tx.mut_repo().view().git_refs(), btreemap! {});
     assert_eq!(*tx.mut_repo().view().branches(), btreemap! {});
 }
@@ -1619,7 +1693,7 @@ fn test_fetch_initial_commit() {
     let mut tx = test_data
         .repo
         .start_transaction(&test_data.settings, "test");
-    let default_branch = git::fetch(
+    let stats = git::fetch(
         tx.mut_repo(),
         &test_data.git_repo,
         "origin",
@@ -1629,7 +1703,8 @@ fn test_fetch_initial_commit() {
     )
     .unwrap();
     // No default branch because the origin repo's HEAD wasn't set
-    assert_eq!(default_branch, None);
+    assert_eq!(stats.default_branch, None);
+    assert!(stats.import_stats.abandoned_commits.is_empty());
     let repo = tx.commit();
     // The initial commit is visible after git::fetch().
     let view = repo.view();
@@ -1680,11 +1755,15 @@ fn test_fetch_success() {
         "refs/heads/main",
         &[&initial_git_commit],
     );
+    test_data
+        .origin_repo
+        .reference("refs/tags/v1.0", new_git_commit.id(), false, "")
+        .unwrap();
 
     let mut tx = test_data
         .repo
         .start_transaction(&test_data.settings, "test");
-    let default_branch = git::fetch(
+    let stats = git::fetch(
         tx.mut_repo(),
         &test_data.git_repo,
         "origin",
@@ -1694,7 +1773,8 @@ fn test_fetch_success() {
     )
     .unwrap();
     // The default branch is "main"
-    assert_eq!(default_branch, Some("main".to_string()));
+    assert_eq!(stats.default_branch, Some("main".to_string()));
+    assert!(stats.import_stats.abandoned_commits.is_empty());
     let repo = tx.commit();
     // The new commit is visible after we fetch again
     let view = repo.view();
@@ -1704,6 +1784,7 @@ fn test_fetch_success() {
         *view.git_refs(),
         btreemap! {
             "refs/remotes/origin/main".to_string() => new_commit_target.clone(),
+            "refs/tags/v1.0".to_string() => new_commit_target.clone(),
         }
     );
     assert_eq!(
@@ -1712,18 +1793,24 @@ fn test_fetch_success() {
             "main".to_string() => BranchTarget {
                 local_target: new_commit_target.clone(),
                 remote_targets: btreemap! {
-                    "origin".to_string() => new_commit_target,
+                    "origin".to_string() => new_commit_target.clone(),
                 },
             },
         }
     );
+    assert_eq!(
+        *view.tags(),
+        btreemap! {
+            "v1.0".to_string() => new_commit_target.clone(),
+        }
+    );
 }
 
 #[test]
 fn test_fetch_prune_deleted_ref() {
     let test_data = GitRepoData::create();
     let git_settings = GitSettings::default();
-    empty_git_commit(&test_data.origin_repo, "refs/heads/main", &[]);
+    let commit = empty_git_commit(&test_data.origin_repo, "refs/heads/main", &[]);
 
     let mut tx = test_data
         .repo
@@ -1747,7 +1834,7 @@ fn test_fetch_prune_deleted_ref() {
         .delete()
         .unwrap();
     // After re-fetching, the branch should be deleted
-    git::fetch(
+    let stats = git::fetch(
         tx.mut_repo(),
         &test_data.git_repo,
         "origin",
@@ -1756,6 +1843,7 @@ fn test_fetch_prune_deleted_ref() {
         &git_settings,
     )
     .unwrap();
+    assert_eq!(stats.import_stats.abandoned_commits, vec![jj_id(&commit)]);
     assert!(tx.mut_repo().get_branch("main").is_none());
 }
 
@@ -1791,7 +1879,7 @@ fn test_fetch_no_default_branch() {
         .set_head_detached(initial_git_commit.id())
         .unwrap();
 
-    let default_branch = git::fetch(
+    let stats = git::fetch(
         tx.mut_repo(),
         &test_data.git_repo,
         "origin",
@@ -1801,7 +1889,7 @@ fn test_fetch_no_default_branch() {
     )
     .unwrap();
     // There is no default branch
-    assert_eq!(default_branch, None);
+    assert_eq!(stats.default_branch, None);
 }
 
 #[test]
@@ -1851,6 +1939,12 @@ fn set_up_push_repos(settings: &UserSettings, temp_dir: &TempDir) -> PushTestSet
         ReadonlyRepo::default_submodule_store_factory(),
     )
     .unwrap();
+    get_git_backend(&jj_repo)
+        .import_head_commits(
+            &[jj_id(&initial_git_commit)],
+            jj_repo.store().use_tree_conflict_format(),
+        )
+        .unwrap();
     let mut tx = jj_repo.start_transaction(settings, "test");
     let new_commit = create_random_commit(tx.mut_repo(), settings)
         .set_parents(vec![jj_id(&initial_git_commit)])
@@ -2062,7 +2156,7 @@ fn test_push_updates_invalid_remote() {
 fn test_bulk_update_extra_on_import_refs() {
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_repo.repo;
     let git_repo = get_git_repo(repo);
 
@@ -2108,7 +2202,7 @@ fn test_bulk_update_extra_on_import_refs() {
 fn test_rewrite_imported_commit() {
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_repo.repo;
     let git_repo = get_git_repo(repo);
 
@@ -2158,7 +2252,7 @@ fn test_rewrite_imported_commit() {
 #[test]
 fn test_concurrent_write_commit() {
     let settings = &testutils::user_settings();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_repo.repo;
 
     // Try to create identical commits with different change ids. Timestamp of the
@@ -2220,7 +2314,7 @@ fn test_concurrent_write_commit() {
 #[test]
 fn test_concurrent_read_write_commit() {
     let settings = &testutils::user_settings();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_repo.repo;
 
     // Create unique commits and load them concurrently. In this test, we assume
@@ -2278,13 +2372,18 @@ fn test_concurrent_read_write_commit() {
                 barrier.wait();
                 while !pending_commit_ids.is_empty() {
                     repo = repo.reload_at_head(settings).unwrap();
+                    let git_backend = get_git_backend(&repo);
                     let mut tx = repo.start_transaction(settings, &format!("reader {i}"));
                     pending_commit_ids = pending_commit_ids
                         .into_iter()
                         .filter_map(|commit_id| {
-                            match repo.store().get_commit(&commit_id) {
-                                Ok(commit) => {
+                            match git_backend.import_head_commits(
+                                [&commit_id],
+                                repo.store().use_tree_conflict_format(),
+                            ) {
+                                Ok(()) => {
                                     // update index as git::import_refs() would do
+                                    let commit = repo.store().get_commit(&commit_id).unwrap();
                                     tx.mut_repo().add_head(&commit);
                                     None
                                 }
