@@ -21,7 +21,7 @@ use testutils::{CommitGraphBuilder, TestWorkspace};
 #[test]
 fn test_merge_ref_targets() {
     let settings = testutils::user_settings();
-    let test_workspace = TestWorkspace::init(&settings, false);
+    let test_workspace = TestWorkspace::init(&settings);
     let repo = &test_workspace.repo;
 
     // 6 7
