@@ -40,7 +40,7 @@ use jj_lib::workspace::Workspace;
 use test_case::test_case;
 use testutils::{
     create_random_commit, create_tree, write_random_commit, CommitGraphBuilder, TestRepo,
-    TestWorkspace,
+    TestRepoBackend, TestWorkspace,
 };
 
 fn resolve_symbol(repo: &dyn Repo, symbol: &str) -> Result<Vec<CommitId>, RevsetResolutionError> {
@@ -72,7 +72,7 @@ fn revset_for_commits<'index>(
 
 #[test]
 fn test_resolve_symbol_empty_string() {
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     assert_matches!(
@@ -85,7 +85,7 @@ fn test_resolve_symbol_empty_string() {
 fn test_resolve_symbol_commit_id() {
     let settings = testutils::user_settings();
     // Test only with git so we can get predictable commit ids
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -193,7 +193,7 @@ fn test_resolve_symbol_change_id(readonly: bool) {
     let settings = testutils::user_settings();
     let git_settings = GitSettings::default();
     // Test only with git so we can get predictable change ids
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
     let repo = &test_repo.repo;
 
     let git_repo = repo
@@ -201,7 +201,8 @@ fn test_resolve_symbol_change_id(readonly: bool) {
         .backend_impl()
         .downcast_ref::<GitBackend>()
         .unwrap()
-        .git_repo_clone();
+        .open_git_repo()
+        .unwrap();
     // Add some commits that will end up having change ids with common prefixes
     let empty_tree_id = git_repo.treebuilder(None).unwrap().write().unwrap();
     let git_author = git2::Signature::new(
@@ -332,11 +333,10 @@ fn test_resolve_symbol_change_id(readonly: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_resolve_working_copy(use_git: bool) {
+#[test]
+fn test_resolve_working_copy() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -380,7 +380,7 @@ fn test_resolve_working_copy(use_git: bool) {
 #[test]
 fn test_resolve_symbol_branches() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -592,7 +592,7 @@ fn test_resolve_symbol_branches() {
 #[test]
 fn test_resolve_symbol_tags() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -619,12 +619,28 @@ fn test_resolve_symbol_tags() {
         resolve_symbol(mut_repo, "unimported"),
         Err(RevsetResolutionError::NoSuchRevision { .. })
     );
+
+    // "@" (quoted) can be resolved, and root is a normal symbol.
+    let ws_id = WorkspaceId::default();
+    mut_repo
+        .set_wc_commit(ws_id.clone(), commit1.id().clone())
+        .unwrap();
+    mut_repo.set_tag_target("@", RefTarget::normal(commit2.id().clone()));
+    mut_repo.set_tag_target("root", RefTarget::normal(commit3.id().clone()));
+    assert_eq!(
+        resolve_symbol(mut_repo, r#""@""#).unwrap(),
+        vec![commit2.id().clone()]
+    );
+    assert_eq!(
+        resolve_symbol(mut_repo, "root").unwrap(),
+        vec![commit3.id().clone()]
+    );
 }
 
 #[test]
 fn test_resolve_symbol_git_head() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -668,7 +684,7 @@ fn test_resolve_symbol_git_head() {
 #[test]
 fn test_resolve_symbol_git_refs() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -753,22 +769,6 @@ fn test_resolve_symbol_git_refs() {
         Err(RevsetResolutionError::NoSuchRevision { .. })
     );
 
-    // "@" (quoted) can be resolved, and root is a normal symbol.
-    let ws_id = WorkspaceId::default();
-    mut_repo
-        .set_wc_commit(ws_id.clone(), commit1.id().clone())
-        .unwrap();
-    mut_repo.set_git_ref_target("@", RefTarget::normal(commit2.id().clone()));
-    mut_repo.set_git_ref_target("root", RefTarget::normal(commit3.id().clone()));
-    assert_eq!(
-        resolve_symbol(mut_repo, r#""@""#).unwrap(),
-        vec![commit2.id().clone()]
-    );
-    assert_eq!(
-        resolve_symbol(mut_repo, "root").unwrap(),
-        vec![commit3.id().clone()]
-    );
-
     // Conflicted ref resolves to its "adds"
     assert_eq!(
         resolve_symbol(mut_repo, "refs/heads/conflicted").unwrap(),
@@ -816,11 +816,10 @@ fn resolve_commit_ids_in_workspace(
     expression.evaluate(repo).unwrap().iter().collect()
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_root_and_checkout(use_git: bool) {
+#[test]
+fn test_evaluate_expression_root_and_checkout() {
     let settings = testutils::user_settings();
-    let test_workspace = TestWorkspace::init(&settings, use_git);
+    let test_workspace = TestWorkspace::init(&settings);
     let repo = &test_workspace.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -845,11 +844,10 @@ fn test_evaluate_expression_root_and_checkout(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_heads(use_git: bool) {
+#[test]
+fn test_evaluate_expression_heads() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let root_commit = repo.store().root_commit();
@@ -911,11 +909,10 @@ fn test_evaluate_expression_heads(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_roots(use_git: bool) {
+#[test]
+fn test_evaluate_expression_roots() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let root_commit = repo.store().root_commit();
@@ -967,11 +964,10 @@ fn test_evaluate_expression_roots(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_parents(use_git: bool) {
+#[test]
+fn test_evaluate_expression_parents() {
     let settings = testutils::user_settings();
-    let test_workspace = TestWorkspace::init(&settings, use_git);
+    let test_workspace = TestWorkspace::init(&settings);
     let repo = &test_workspace.repo;
 
     let root_commit = repo.store().root_commit();
@@ -1050,11 +1046,10 @@ fn test_evaluate_expression_parents(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_children(use_git: bool) {
+#[test]
+fn test_evaluate_expression_children() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -1137,11 +1132,10 @@ fn test_evaluate_expression_children(use_git: bool) {
     assert_eq!(resolve_commit_ids(mut_repo, "none()+"), vec![]);
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_ancestors(use_git: bool) {
+#[test]
+fn test_evaluate_expression_ancestors() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let root_commit = repo.store().root_commit();
@@ -1205,13 +1199,30 @@ fn test_evaluate_expression_ancestors(use_git: bool) {
             root_commit.id().clone(),
         ]
     );
+
+    // Can find last n ancestors of a commit
+    assert_eq!(
+        resolve_commit_ids(mut_repo, &format!("ancestors({}, 0)", commit2.id().hex())),
+        vec![]
+    );
+    assert_eq!(
+        resolve_commit_ids(mut_repo, &format!("ancestors({}, 1)", commit3.id().hex())),
+        vec![commit3.id().clone()]
+    );
+    assert_eq!(
+        resolve_commit_ids(mut_repo, &format!("ancestors({}, 3)", commit3.id().hex())),
+        vec![
+            commit3.id().clone(),
+            commit2.id().clone(),
+            commit1.id().clone(),
+        ]
+    );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_range(use_git: bool) {
+#[test]
+fn test_evaluate_expression_range() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -1289,11 +1300,10 @@ fn test_evaluate_expression_range(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_dag_range(use_git: bool) {
+#[test]
+fn test_evaluate_expression_dag_range() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let root_commit_id = repo.store().root_commit_id().clone();
@@ -1400,11 +1410,10 @@ fn test_evaluate_expression_dag_range(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_connected(use_git: bool) {
+#[test]
+fn test_evaluate_expression_connected() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let root_commit_id = repo.store().root_commit_id().clone();
@@ -1477,11 +1486,10 @@ fn test_evaluate_expression_connected(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_descendants(use_git: bool) {
+#[test]
+fn test_evaluate_expression_descendants() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -1579,21 +1587,19 @@ fn test_evaluate_expression_descendants(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_none(use_git: bool) {
-    let test_repo = TestRepo::init(use_git);
+#[test]
+fn test_evaluate_expression_none() {
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     // none() is empty (doesn't include the checkout, for example)
     assert_eq!(resolve_commit_ids(repo.as_ref(), "none()"), vec![]);
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_all(use_git: bool) {
+#[test]
+fn test_evaluate_expression_all() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -1617,11 +1623,10 @@ fn test_evaluate_expression_all(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_visible_heads(use_git: bool) {
+#[test]
+fn test_evaluate_expression_visible_heads() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -1637,11 +1642,10 @@ fn test_evaluate_expression_visible_heads(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_git_refs(use_git: bool) {
+#[test]
+fn test_evaluate_expression_git_refs() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -1697,11 +1701,10 @@ fn test_evaluate_expression_git_refs(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_git_head(use_git: bool) {
+#[test]
+fn test_evaluate_expression_git_head() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -1718,11 +1721,10 @@ fn test_evaluate_expression_git_head(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_branches(use_git: bool) {
+#[test]
+fn test_evaluate_expression_branches() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -1794,11 +1796,10 @@ fn test_evaluate_expression_branches(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_remote_branches(use_git: bool) {
+#[test]
+fn test_evaluate_expression_remote_branches() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -1923,11 +1924,10 @@ fn test_evaluate_expression_remote_branches(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_latest(use_git: bool) {
+#[test]
+fn test_evaluate_expression_latest() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -2007,11 +2007,10 @@ fn test_evaluate_expression_latest(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_merges(use_git: bool) {
+#[test]
+fn test_evaluate_expression_merges() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -2035,11 +2034,10 @@ fn test_evaluate_expression_merges(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_description(use_git: bool) {
+#[test]
+fn test_evaluate_expression_description() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -2081,11 +2079,10 @@ fn test_evaluate_expression_description(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_author(use_git: bool) {
+#[test]
+fn test_evaluate_expression_author() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -2155,11 +2152,10 @@ fn test_evaluate_expression_author(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_mine(use_git: bool) {
+#[test]
+fn test_evaluate_expression_mine() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -2224,11 +2220,10 @@ fn test_evaluate_expression_mine(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_committer(use_git: bool) {
+#[test]
+fn test_evaluate_expression_committer() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -2290,11 +2285,10 @@ fn test_evaluate_expression_committer(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_union(use_git: bool) {
+#[test]
+fn test_evaluate_expression_union() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let root_commit = repo.store().root_commit();
@@ -2363,11 +2357,10 @@ fn test_evaluate_expression_union(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_intersection(use_git: bool) {
+#[test]
+fn test_evaluate_expression_intersection() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let root_commit = repo.store().root_commit();
@@ -2403,11 +2396,10 @@ fn test_evaluate_expression_intersection(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_difference(use_git: bool) {
+#[test]
+fn test_evaluate_expression_difference() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let root_commit = repo.store().root_commit();
@@ -2490,11 +2482,10 @@ fn test_evaluate_expression_difference(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_filter_combinator(use_git: bool) {
+#[test]
+fn test_evaluate_expression_filter_combinator() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -2554,11 +2545,10 @@ fn test_evaluate_expression_filter_combinator(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_file(use_git: bool) {
+#[test]
+fn test_evaluate_expression_file() {
     let settings = testutils::user_settings();
-    let test_workspace = TestWorkspace::init(&settings, use_git);
+    let test_workspace = TestWorkspace::init(&settings);
     let repo = &test_workspace.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -2665,11 +2655,10 @@ fn test_evaluate_expression_file(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_evaluate_expression_conflict(use_git: bool) {
+#[test]
+fn test_evaluate_expression_conflict() {
     let settings = testutils::user_settings();
-    let test_workspace = TestWorkspace::init(&settings, use_git);
+    let test_workspace = TestWorkspace::init(&settings);
     let repo = &test_workspace.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -2704,7 +2693,7 @@ fn test_evaluate_expression_conflict(use_git: bool) {
 #[test]
 fn test_reverse_graph_iterator() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     // Tests that merges, forks, direct edges, indirect edges, and "missing" edges
@@ -2769,7 +2758,7 @@ fn test_reverse_graph_iterator() {
 #[test]
 fn test_change_id_index() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -2875,7 +2864,7 @@ fn test_change_id_index() {
 #[test]
 fn test_no_such_revision_suggestion() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(true);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
