@@ -20,10 +20,9 @@ use testutils::{
     commit_transactions, create_random_commit, write_random_commit, CommitGraphBuilder, TestRepo,
 };
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_heads_empty(use_git: bool) {
-    let test_repo = TestRepo::init(use_git);
+#[test]
+fn test_heads_empty() {
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     assert_eq!(
@@ -36,11 +35,10 @@ fn test_heads_empty(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_heads_fork(use_git: bool) {
+#[test]
+fn test_heads_fork() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
     let mut tx = repo.start_transaction(&settings, "test");
 
@@ -59,11 +57,10 @@ fn test_heads_fork(use_git: bool) {
     );
 }
 
-#[test_case(false ; "local backend")]
-#[test_case(true ; "git backend")]
-fn test_heads_merge(use_git: bool) {
+#[test]
+fn test_heads_merge() {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
     let mut tx = repo.start_transaction(&settings, "test");
 
@@ -81,7 +78,7 @@ fn test_heads_merge(use_git: bool) {
 fn test_merge_views_heads() {
     // Tests merging of the view's heads (by performing concurrent operations).
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(false);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -139,7 +136,7 @@ fn test_merge_views_heads() {
 fn test_merge_views_checkout() {
     // Tests merging of the view's checkout (by performing concurrent operations).
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(false);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     // Workspace 1 gets updated in both transactions.
@@ -230,7 +227,7 @@ fn test_merge_views_branches() {
     // Tests merging of branches (by performing concurrent operations). See
     // test_refs.rs for tests of merging of individual ref targets.
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(false);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -254,7 +251,7 @@ fn test_merge_views_branches() {
         RefTarget::normal(main_branch_alternate_tx0.id().clone()),
     );
     let feature_branch_local_tx0 = write_random_commit(mut_repo, &settings);
-    mut_repo.set_git_ref_target(
+    mut_repo.set_local_branch_target(
         "feature",
         RefTarget::normal(feature_branch_local_tx0.id().clone()),
     );
@@ -321,7 +318,7 @@ fn test_merge_views_tags() {
     // Tests merging of tags (by performing concurrent operations). See
     // test_refs.rs for tests of merging of individual ref targets.
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(false);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -365,7 +362,7 @@ fn test_merge_views_git_refs() {
     // Tests merging of git refs (by performing concurrent operations). See
     // test_refs.rs for tests of merging of individual ref targets.
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(false);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction(&settings, "test");
@@ -421,7 +418,7 @@ fn test_merge_views_git_heads() {
     // Tests merging of git heads (by performing concurrent operations). See
     // test_refs.rs for tests of merging of individual ref targets.
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(false);
+    let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
     let mut tx0 = repo.start_transaction(&settings, "test");
@@ -453,7 +450,7 @@ fn test_merge_views_divergent() {
     // We start with just commit A. Operation 1 rewrites it as A2. Operation 2
     // rewrites it as A3.
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(false);
+    let test_repo = TestRepo::init();
 
     let mut tx = test_repo.repo.start_transaction(&settings, "test");
     let commit_a = write_random_commit(tx.mut_repo(), &settings);
@@ -492,7 +489,7 @@ fn test_merge_views_child_on_rewritten(child_first: bool) {
     // We start with just commit A. Operation 1 adds commit B on top. Operation 2
     // rewrites A as A2.
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(false);
+    let test_repo = TestRepo::init();
 
     let mut tx = test_repo.repo.start_transaction(&settings, "test");
     let commit_a = write_random_commit(tx.mut_repo(), &settings);
@@ -538,7 +535,7 @@ fn test_merge_views_child_on_rewritten_divergent(on_rewritten: bool, child_first
     // gets rebased onto A4 if it was based on A2 before, but if it was based on
     // A3, it should remain there.
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(false);
+    let test_repo = TestRepo::init();
 
     let mut tx = test_repo.repo.start_transaction(&settings, "test");
     let commit_a2 = write_random_commit(tx.mut_repo(), &settings);
@@ -594,7 +591,7 @@ fn test_merge_views_child_on_abandoned(child_first: bool) {
     // We start with commit B on top of commit A. Operation 1 adds commit C on top.
     // Operation 2 abandons B.
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(false);
+    let test_repo = TestRepo::init();
 
     let mut tx = test_repo.repo.start_transaction(&settings, "test");
     let commit_a = write_random_commit(tx.mut_repo(), &settings);
