@@ -36,6 +36,10 @@ use jj_lib::working_copy::{SnapshotError, SnapshotOptions};
 use jj_lib::workspace::Workspace;
 use tempfile::TempDir;
 
+use crate::test_backend::TestBackend;
+
+pub mod test_backend;
+
 pub fn hermetic_libgit2() {
     // libgit2 respects init.defaultBranch (and possibly other config
     // variables) in the user's config files. Disable access to them to make
@@ -84,49 +88,62 @@ pub struct TestRepo {
     pub repo: Arc<ReadonlyRepo>,
 }
 
+#[derive(PartialEq, Eq, Copy, Clone)]
+pub enum TestRepoBackend {
+    Git,
+    Local,
+    Test,
+}
+
+impl TestRepoBackend {
+    fn init_backend(&self, store_path: &Path) -> Result<Box<dyn Backend>, BackendInitError> {
+        match self {
+            TestRepoBackend::Git => Ok(Box::new(GitBackend::init_internal(store_path)?)),
+            TestRepoBackend::Local => Ok(Box::new(LocalBackend::init(store_path))),
+            TestRepoBackend::Test => Ok(Box::new(TestBackend::init(store_path))),
+        }
+    }
+}
+
 impl TestRepo {
-    pub fn init(use_git: bool) -> Self {
+    pub fn init() -> Self {
+        Self::init_with_backend(TestRepoBackend::Test)
+    }
+
+    pub fn init_with_backend(backend: TestRepoBackend) -> Self {
         let settings = user_settings();
         let temp_dir = new_temp_dir();
 
         let repo_dir = temp_dir.path().join("repo");
         fs::create_dir(&repo_dir).unwrap();
 
-        let repo = if use_git {
-            let git_path = temp_dir.path().join("git-repo");
-            git2::Repository::init(&git_path).unwrap();
-            ReadonlyRepo::init(
-                &settings,
-                &repo_dir,
-                |store_path| -> Result<Box<dyn Backend>, BackendInitError> {
-                    Ok(Box::new(GitBackend::init_external(store_path, &git_path)?))
-                },
-                ReadonlyRepo::default_op_store_factory(),
-                ReadonlyRepo::default_op_heads_store_factory(),
-                ReadonlyRepo::default_index_store_factory(),
-                ReadonlyRepo::default_submodule_store_factory(),
-            )
-            .unwrap()
-        } else {
-            ReadonlyRepo::init(
-                &settings,
-                &repo_dir,
-                |store_path| -> Result<Box<dyn Backend>, BackendInitError> {
-                    Ok(Box::new(LocalBackend::init(store_path)))
-                },
-                ReadonlyRepo::default_op_store_factory(),
-                ReadonlyRepo::default_op_heads_store_factory(),
-                ReadonlyRepo::default_index_store_factory(),
-                ReadonlyRepo::default_submodule_store_factory(),
-            )
-            .unwrap()
-        };
+        let repo = ReadonlyRepo::init(
+            &settings,
+            &repo_dir,
+            |store_path| -> Result<Box<dyn Backend>, BackendInitError> {
+                backend.init_backend(store_path)
+            },
+            ReadonlyRepo::default_op_store_factory(),
+            ReadonlyRepo::default_op_heads_store_factory(),
+            ReadonlyRepo::default_index_store_factory(),
+            ReadonlyRepo::default_submodule_store_factory(),
+        )
+        .unwrap();
 
         Self {
             _temp_dir: temp_dir,
             repo,
         }
     }
+
+    pub fn default_store_factories() -> StoreFactories {
+        let mut factories = StoreFactories::default();
+        factories.add_backend(
+            "test",
+            Box::new(|store_path| Ok(Box::new(TestBackend::load(store_path)))),
+        );
+        factories
+    }
 }
 
 pub struct TestWorkspace {
@@ -137,19 +154,21 @@ pub struct TestWorkspace {
 }
 
 impl TestWorkspace {
-    pub fn init(settings: &UserSettings, use_git: bool) -> Self {
+    pub fn init(settings: &UserSettings) -> Self {
+        Self::init_with_backend(settings, TestRepoBackend::Test)
+    }
+
+    pub fn init_with_backend(settings: &UserSettings, backend: TestRepoBackend) -> Self {
         let temp_dir = new_temp_dir();
 
         let workspace_root = temp_dir.path().join("repo");
         fs::create_dir(&workspace_root).unwrap();
 
-        let (workspace, repo) = if use_git {
-            let git_path = temp_dir.path().join("git-repo");
-            git2::Repository::init(&git_path).unwrap();
-            Workspace::init_external_git(settings, &workspace_root, &git_path).unwrap()
-        } else {
-            Workspace::init_local(settings, &workspace_root).unwrap()
-        };
+        let (workspace, repo) =
+            Workspace::init_with_backend(settings, &workspace_root, |store_path| {
+                backend.init_backend(store_path)
+            })
+            .unwrap();
 
         Self {
             temp_dir,
@@ -179,7 +198,7 @@ impl TestWorkspace {
 }
 
 pub fn load_repo_at_head(settings: &UserSettings, repo_path: &Path) -> Arc<ReadonlyRepo> {
-    RepoLoader::init(settings, repo_path, &StoreFactories::default())
+    RepoLoader::init(settings, repo_path, &TestRepo::default_store_factories())
         .unwrap()
         .load_at_head(settings)
         .unwrap()
